@@ -2,10 +2,12 @@
 """Implements a class to be used for unit testing.
 """
 import datetime
+import os
 from tlsmate.cert_chain import CertChain
 from tlsmate.cert import Certificate
-from tlsmate.exception import UntrustedCertificate
+from tlsmate.tls import UntrustedCertificate
 from tlsmate import tls
+from tlsmate import tlsmate as tm
 from cryptography.x509 import ocsp, load_pem_x509_certificate, ReasonFlags
 from cryptography.hazmat.primitives.serialization import Encoding
 from cryptography.hazmat.primitives import hashes, serialization
@@ -14,7 +16,33 @@ import pytest
 import requests
 
 
-def test_revoked_certificate(tlsmate, valid_time, server_revoked_rsa_cert, ca_rsa_cert):
+def init_crl(ca_dir, crl_manager, ca):
+    port = 44400
+    if "TLSMATE_CA_PORT" in os.environ:
+        port = os.environ["TLSMATE_CA_PORT"]
+
+    pem_file = ca_dir / f"crl/{ca}.crl.pem"
+    with open(pem_file, "rb") as fd:
+        crl = fd.read()
+    crl_manager.add_crl(f"http://crl.localhost:{port}/crl/{ca}.crl", pem_crl=crl)
+
+
+@pytest.fixture
+def tlsmate_cert(ca_dir, trust_store_file):
+    mate = tm.TlsMate()
+    mate.trust_store.set_ca_files([trust_store_file])
+    init_crl(ca_dir, mate.crl_manager, "root-rsa")
+    init_crl(ca_dir, mate.crl_manager, "root-ecdsa")
+    init_crl(ca_dir, mate.crl_manager, "ca-rsa")
+    init_crl(ca_dir, mate.crl_manager, "ca-ecdsa")
+    mate.config.set("ocsp", False)
+    mate.config.set("crl", True)
+    return mate
+
+
+def test_revoked_certificate(
+    tlsmate_cert, valid_time, server_revoked_rsa_cert, ca_rsa_cert
+):
 
     chain = CertChain()
     for cert in (server_revoked_rsa_cert, ca_rsa_cert):
@@ -24,7 +52,7 @@ def test_revoked_certificate(tlsmate, valid_time, server_revoked_rsa_cert, ca_rs
         chain.validate(valid_time, "revoked.localhost", True)
 
 
-def test_certificate_not_yet_valid(tlsmate, server_rsa_cert, ca_rsa_cert):
+def test_certificate_not_yet_valid(tlsmate_cert, server_rsa_cert, ca_rsa_cert):
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
@@ -34,7 +62,7 @@ def test_certificate_not_yet_valid(tlsmate, server_rsa_cert, ca_rsa_cert):
         chain.validate(datetime.datetime(2000, 2, 27), "localhost", True)
 
 
-def test_certificate_expired(tlsmate, server_rsa_cert, ca_rsa_cert):
+def test_certificate_expired(tlsmate_cert, server_rsa_cert, ca_rsa_cert):
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
@@ -44,7 +72,7 @@ def test_certificate_expired(tlsmate, server_rsa_cert, ca_rsa_cert):
         chain.validate(datetime.datetime(2200, 2, 27), "localhost", True)
 
 
-def test_dsa_certificate(tlsmate, valid_time, server_dsa_cert, ca_rsa_cert):
+def test_dsa_certificate(tlsmate_cert, valid_time, server_dsa_cert, ca_rsa_cert):
 
     chain = CertChain()
     for cert in (server_dsa_cert, ca_rsa_cert):
@@ -54,7 +82,9 @@ def test_dsa_certificate(tlsmate, valid_time, server_dsa_cert, ca_rsa_cert):
     assert True
 
 
-def test_ed25519_certificate(tlsmate, valid_time, server_ed25519_cert, ca_ecdsa_cert):
+def test_ed25519_certificate(
+    tlsmate_cert, valid_time, server_ed25519_cert, ca_ecdsa_cert
+):
 
     chain = CertChain()
     for cert in (server_ed25519_cert, ca_ecdsa_cert):
@@ -64,7 +94,7 @@ def test_ed25519_certificate(tlsmate, valid_time, server_ed25519_cert, ca_ecdsa_
     assert True
 
 
-def test_ed448_certificate(tlsmate, valid_time, server_ed448_cert, ca_ecdsa_cert):
+def test_ed448_certificate(tlsmate_cert, valid_time, server_ed448_cert, ca_ecdsa_cert):
 
     chain = CertChain()
     for cert in (server_ed448_cert, ca_ecdsa_cert):
@@ -75,7 +105,7 @@ def test_ed448_certificate(tlsmate, valid_time, server_ed448_cert, ca_ecdsa_cert
 
 
 def test_rsa_with_root_certificate(
-    tlsmate, valid_time, server_rsa_cert, ca_rsa_cert, root_rsa_cert
+    tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert, root_rsa_cert
 ):
 
     chain = CertChain()
@@ -86,7 +116,7 @@ def test_rsa_with_root_certificate(
     assert True
 
 
-def test_wrong_sni(tlsmate, valid_time, server_rsa_cert, ca_rsa_cert):
+def test_wrong_sni(tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert):
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
@@ -97,7 +127,7 @@ def test_wrong_sni(tlsmate, valid_time, server_rsa_cert, ca_rsa_cert):
 
 
 def test_root_not_last_in_chain(
-    tlsmate, valid_time, server_rsa_cert, ca_rsa_cert, ca_ecdsa_cert, root_rsa_cert
+    tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert, ca_ecdsa_cert, root_rsa_cert
 ):
 
     chain = CertChain()
@@ -108,7 +138,7 @@ def test_root_not_last_in_chain(
     assert True
 
 
-def test_issuer_mismatch(tlsmate, valid_time, server_rsa_cert, ca_ecdsa_cert):
+def test_issuer_mismatch(tlsmate_cert, valid_time, server_rsa_cert, ca_ecdsa_cert):
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_ecdsa_cert):
@@ -118,7 +148,9 @@ def test_issuer_mismatch(tlsmate, valid_time, server_rsa_cert, ca_ecdsa_cert):
         chain.validate(valid_time, "localhost", True)
 
 
-def test_signature_invalid_chain(tlsmate, valid_time, server_rsa_cert, ca_2nd_rsa_cert):
+def test_signature_invalid_chain(
+    tlsmate_cert, valid_time, server_rsa_cert, ca_2nd_rsa_cert
+):
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_2nd_rsa_cert):
@@ -129,11 +161,11 @@ def test_signature_invalid_chain(tlsmate, valid_time, server_rsa_cert, ca_2nd_rs
 
 
 def test_root_in_chain_not_in_truststore(
-    tlsmate, valid_time, server_rsa_cert, ca_rsa_cert, root_rsa_cert
+    tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert, root_rsa_cert
 ):
 
     # hard reset of the trust store
-    tlsmate.trust_store._ca_files = None
+    tlsmate_cert.trust_store._ca_files = None
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert, root_rsa_cert):
@@ -146,11 +178,11 @@ def test_root_in_chain_not_in_truststore(
 
 
 def test_root_not_in_chain_not_in_truststore(
-    tlsmate, valid_time, server_rsa_cert, ca_rsa_cert
+    tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert
 ):
 
     # hard reset of the trust store
-    tlsmate.trust_store._ca_files = None
+    tlsmate_cert.trust_store._ca_files = None
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
@@ -160,7 +192,7 @@ def test_root_not_in_chain_not_in_truststore(
         chain.validate(valid_time, "localhost", True)
 
 
-def test_rsa_san(tlsmate, valid_time, server_rsa_cert, ca_rsa_cert):
+def test_rsa_san(tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert):
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
@@ -174,7 +206,7 @@ def test_rsa_san(tlsmate, valid_time, server_rsa_cert, ca_rsa_cert):
 
 
 def test_certs_not_in_sequence(
-    tlsmate, valid_time, server_rsa_cert, ca_rsa_cert, root_rsa_cert
+    tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert, root_rsa_cert
 ):
 
     chain = CertChain()
@@ -186,7 +218,7 @@ def test_certs_not_in_sequence(
 
 
 def test_gratuitous_certificate(
-    tlsmate, valid_time, server_rsa_cert, ca_rsa_cert, ca_ecdsa_cert
+    tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert, ca_ecdsa_cert
 ):
 
     chain = CertChain()
@@ -201,11 +233,11 @@ def test_gratuitous_certificate(
 
 
 def test_root_not_in_chain_not_in_truststore_no_exception(
-    tlsmate, valid_time, server_rsa_cert, ca_rsa_cert
+    tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert
 ):
 
     # hard reset of the trust store
-    tlsmate.trust_store._ca_files = None
+    tlsmate_cert.trust_store._ca_files = None
 
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
@@ -220,12 +252,12 @@ def requests_post_timeout(url, **kwargs):
 
 
 def test_ocsp_status_timeout(
-    monkeypatch, tlsmate, valid_time, server_rsa_cert, ca_rsa_cert
+    monkeypatch, tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert
 ):
 
     monkeypatch.setattr(requests, "post", requests_post_timeout)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -241,12 +273,12 @@ def requests_post_exception(url, **kwargs):
 
 
 def test_ocsp_status_exception(
-    monkeypatch, tlsmate, valid_time, server_rsa_cert, ca_rsa_cert
+    monkeypatch, tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert
 ):
 
     monkeypatch.setattr(requests, "post", requests_post_exception)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -258,12 +290,12 @@ def test_ocsp_status_exception(
 
 
 def test_ocsp_status_cached_chain(
-    monkeypatch, tlsmate, valid_time, server_rsa_cert, ca_rsa_cert
+    monkeypatch, tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert
 ):
 
     monkeypatch.setattr(requests, "post", requests_post_exception)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -297,7 +329,7 @@ def validate_ocsp(self, cert, issuer_cert, timestamp):
 
 
 def test_ocsp_status_invalid_signature(
-    monkeypatch, tlsmate, server_rsa_cert, ca_rsa_cert, root_rsa_cert, root_rsa_key
+    monkeypatch, tlsmate_cert, server_rsa_cert, ca_rsa_cert, root_rsa_cert, root_rsa_key
 ):
 
     now = datetime.datetime.now()
@@ -328,7 +360,7 @@ def test_ocsp_status_invalid_signature(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -340,7 +372,7 @@ def test_ocsp_status_invalid_signature(
 
 
 def test_ocsp_invalid_reponse(
-    monkeypatch, tlsmate, server_rsa_cert, ca_rsa_cert,
+    monkeypatch, tlsmate_cert, server_rsa_cert, ca_rsa_cert,
 ):
 
     now = datetime.datetime.now()
@@ -354,7 +386,7 @@ def test_ocsp_invalid_reponse(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -366,7 +398,7 @@ def test_ocsp_invalid_reponse(
 
 
 def test_ocsp_status_with_cert(
-    monkeypatch, tlsmate, server_rsa_cert, ca_rsa_cert, ca_rsa_key
+    monkeypatch, tlsmate_cert, server_rsa_cert, ca_rsa_cert, ca_rsa_key
 ):
 
     now = datetime.datetime.now()
@@ -398,7 +430,7 @@ def test_ocsp_status_with_cert(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -409,7 +441,7 @@ def test_ocsp_status_with_cert(
 
 def test_ocsp_status_with_invalid_cert(
     monkeypatch,
-    tlsmate,
+    tlsmate_cert,
     server_rsa_cert,
     ca_rsa_cert,
     ca_rsa_key,
@@ -446,7 +478,7 @@ def test_ocsp_status_with_invalid_cert(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -458,7 +490,7 @@ def test_ocsp_status_with_invalid_cert(
 
 
 def test_ocsp_status_invalid_update_time(
-    monkeypatch, tlsmate, server_rsa_cert, ca_rsa_cert, ca_rsa_key
+    monkeypatch, tlsmate_cert, server_rsa_cert, ca_rsa_cert, ca_rsa_key
 ):
 
     now = datetime.datetime.now()
@@ -490,7 +522,7 @@ def test_ocsp_status_invalid_update_time(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -502,7 +534,7 @@ def test_ocsp_status_invalid_update_time(
 
 
 def test_ocsp_status_invalid_next_update(
-    monkeypatch, tlsmate, server_rsa_cert, ca_rsa_cert, ca_rsa_key
+    monkeypatch, tlsmate_cert, server_rsa_cert, ca_rsa_cert, ca_rsa_key
 ):
 
     now = datetime.datetime.now()
@@ -534,7 +566,7 @@ def test_ocsp_status_invalid_next_update(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -546,7 +578,7 @@ def test_ocsp_status_invalid_next_update(
 
 
 def test_ocsp_invalid_response(
-    monkeypatch, tlsmate, server_rsa_cert, ca_rsa_cert, ca_rsa_key
+    monkeypatch, tlsmate_cert, server_rsa_cert, ca_rsa_cert, ca_rsa_key
 ):
 
     now = datetime.datetime.now()
@@ -560,7 +592,7 @@ def test_ocsp_invalid_response(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -572,7 +604,7 @@ def test_ocsp_invalid_response(
 
 
 def test_ocsp_status_revoked(
-    monkeypatch, tlsmate, server_rsa_cert, ca_rsa_cert, ca_rsa_key
+    monkeypatch, tlsmate_cert, server_rsa_cert, ca_rsa_cert, ca_rsa_key
 ):
 
     now = datetime.datetime.now()
@@ -604,7 +636,7 @@ def test_ocsp_status_revoked(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -614,7 +646,7 @@ def test_ocsp_status_revoked(
 
 
 def test_ocsp_status_unknown(
-    monkeypatch, tlsmate, server_rsa_cert, ca_rsa_cert, ca_rsa_key
+    monkeypatch, tlsmate_cert, server_rsa_cert, ca_rsa_cert, ca_rsa_key
 ):
 
     now = datetime.datetime.now()
@@ -646,7 +678,7 @@ def test_ocsp_status_unknown(
     monkeypatch.setattr(requests, "post", requests_posts)
     monkeypatch.setattr(CertChain, "_valid_ocsp", validate_ocsp)
 
-    tlsmate.config.set("ocsp", True)
+    tlsmate_cert.config.set("ocsp", True)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
@@ -655,7 +687,7 @@ def test_ocsp_status_unknown(
         chain.validate(now, "localhost", True)
 
 
-def test_cert_no_ids(tlsmate, valid_time, server_no_ids_rsa_cert, ca_rsa_cert):
+def test_cert_no_ids(tlsmate_cert, valid_time, server_no_ids_rsa_cert, ca_rsa_cert):
 
     chain = CertChain()
     for cert in (server_no_ids_rsa_cert, ca_rsa_cert):
@@ -666,7 +698,7 @@ def test_cert_no_ids(tlsmate, valid_time, server_no_ids_rsa_cert, ca_rsa_cert):
 
 
 def test_cert_invalid_signature(
-    monkeypatch, tlsmate, valid_time, server_rsa_cert, ca_rsa_cert
+    monkeypatch, tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert
 ):
     def validate_cert_signature(cert):
         raise Exception("bla bla")
@@ -682,7 +714,7 @@ def test_cert_invalid_signature(
 
 
 def test_cert_untrusted_trust_path(
-    tlsmate, valid_time, server_rsa_cert, ca_rsa_cert, ca_ecdsa_cert
+    tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert, ca_ecdsa_cert
 ):
 
     chain = CertChain()
@@ -697,9 +729,9 @@ def test_cert_untrusted_trust_path(
     )
 
 
-def test_no_crl(tlsmate, valid_time, server_rsa_cert, ca_rsa_cert):
+def test_no_crl(tlsmate_cert, valid_time, server_rsa_cert, ca_rsa_cert):
 
-    tlsmate.config.set("crl", False)
+    tlsmate_cert.config.set("crl", False)
     chain = CertChain()
     for cert in (server_rsa_cert, ca_rsa_cert):
         chain.append_pem_cert(cert.as_bytes())
