@@ -7,72 +7,90 @@ from tlsmate.tlssuite import TlsSuiteTester
 from tlsmate.tlssuite import TlsLibrary
 
 
-tls12_cs = [
+tls10_cs = [
+    "TLS_RSA_WITH_RC4_128_MD5",
+    "TLS_RSA_WITH_RC4_128_SHA",
+    "TLS_RSA_WITH_IDEA_CBC_SHA",
+    "TLS_RSA_WITH_3DES_EDE_CBC_SHA",
+    "TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA",
+    "TLS_DH_ANON_WITH_RC4_128_MD5",
+    "TLS_DH_ANON_WITH_3DES_EDE_CBC_SHA",
     "TLS_RSA_WITH_AES_128_CBC_SHA",
     "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
+    "TLS_DH_ANON_WITH_AES_128_CBC_SHA",
     "TLS_RSA_WITH_AES_256_CBC_SHA",
     "TLS_DHE_RSA_WITH_AES_256_CBC_SHA",
+    "TLS_DH_ANON_WITH_AES_256_CBC_SHA",
+    "TLS_RSA_WITH_CAMELLIA_128_CBC_SHA",
+    "TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA",
+    "TLS_DH_ANON_WITH_CAMELLIA_128_CBC_SHA",
+    "TLS_RSA_WITH_CAMELLIA_256_CBC_SHA",
+    "TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA",
+    "TLS_DH_ANON_WITH_CAMELLIA_256_CBC_SHA",
+    "TLS_RSA_WITH_SEED_CBC_SHA",
+    "TLS_DHE_RSA_WITH_SEED_CBC_SHA",
+    "TLS_DH_ANON_WITH_SEED_CBC_SHA",
+    "TLS_ECDH_ECDSA_WITH_RC4_128_SHA",
+    "TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA",
+    "TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA",
+    "TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA",
+    "TLS_ECDHE_ECDSA_WITH_RC4_128_SHA",
+    "TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA",
+    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA",
+    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA",
+    "TLS_ECDHE_RSA_WITH_RC4_128_SHA",
+    "TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA",
+    "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA",
+    "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA",
+]
+
+tls12_cs = [
+    "TLS_RSA_WITH_RC4_128_MD5",
+    "TLS_RSA_WITH_RC4_128_SHA",
+    "TLS_RSA_WITH_IDEA_CBC_SHA",
+    "TLS_RSA_WITH_3DES_EDE_CBC_SHA",
+    "TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA",
+    "TLS_DH_ANON_WITH_RC4_128_MD5",
+    "TLS_DH_ANON_WITH_3DES_EDE_CBC_SHA",
+    "TLS_RSA_WITH_AES_128_CBC_SHA",
+    "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
+    "TLS_DH_ANON_WITH_AES_128_CBC_SHA",
+    "TLS_RSA_WITH_AES_256_CBC_SHA",
+    "TLS_DHE_RSA_WITH_AES_256_CBC_SHA",
+    "TLS_DH_ANON_WITH_AES_256_CBC_SHA",
     "TLS_RSA_WITH_AES_128_CBC_SHA256",
     "TLS_RSA_WITH_AES_256_CBC_SHA256",
     "TLS_RSA_WITH_CAMELLIA_128_CBC_SHA",
     "TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA",
+    "TLS_DH_ANON_WITH_CAMELLIA_128_CBC_SHA",
     "TLS_DHE_RSA_WITH_AES_128_CBC_SHA256",
     "TLS_DHE_RSA_WITH_AES_256_CBC_SHA256",
+    "TLS_DH_ANON_WITH_AES_128_CBC_SHA256",
+    "TLS_DH_ANON_WITH_AES_256_CBC_SHA256",
     "TLS_RSA_WITH_CAMELLIA_256_CBC_SHA",
     "TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA",
+    "TLS_DH_ANON_WITH_CAMELLIA_256_CBC_SHA",
+    "TLS_RSA_WITH_SEED_CBC_SHA",
+    "TLS_DHE_RSA_WITH_SEED_CBC_SHA",
+    "TLS_DH_ANON_WITH_SEED_CBC_SHA",
     "TLS_RSA_WITH_AES_128_GCM_SHA256",
     "TLS_RSA_WITH_AES_256_GCM_SHA384",
     "TLS_DHE_RSA_WITH_AES_128_GCM_SHA256",
     "TLS_DHE_RSA_WITH_AES_256_GCM_SHA384",
-    "TLS_RSA_WITH_CAMELLIA_128_CBC_SHA256",
-    "TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA256",
-    "TLS_RSA_WITH_CAMELLIA_256_CBC_SHA256",
-    "TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA256",
+    "TLS_DH_ANON_WITH_AES_128_GCM_SHA256",
+    "TLS_DH_ANON_WITH_AES_256_GCM_SHA384",
+    "TLS_ECDH_ECDSA_WITH_RC4_128_SHA",
+    "TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA",
+    "TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA",
+    "TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA",
+    "TLS_ECDHE_ECDSA_WITH_RC4_128_SHA",
+    "TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA",
     "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA",
     "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA",
+    "TLS_ECDHE_RSA_WITH_RC4_128_SHA",
+    "TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA",
     "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA",
     "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA",
-    "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",
-    "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384",
-    "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",
-    "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384",
-    "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256",
-    "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384",
-    "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256",
-    "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384",
-    "TLS_RSA_WITH_ARIA_128_GCM_SHA256",
-    "TLS_RSA_WITH_ARIA_256_GCM_SHA384",
-    "TLS_DHE_RSA_WITH_ARIA_128_GCM_SHA256",
-    "TLS_DHE_RSA_WITH_ARIA_256_GCM_SHA384",
-    "TLS_ECDHE_ECDSA_WITH_ARIA_128_GCM_SHA256",
-    "TLS_ECDHE_ECDSA_WITH_ARIA_256_GCM_SHA384",
-    "TLS_ECDHE_RSA_WITH_ARIA_128_GCM_SHA256",
-    "TLS_ECDHE_RSA_WITH_ARIA_256_GCM_SHA384",
-    "TLS_ECDHE_ECDSA_WITH_CAMELLIA_128_CBC_SHA256",
-    "TLS_ECDHE_ECDSA_WITH_CAMELLIA_256_CBC_SHA384",
-    "TLS_ECDHE_RSA_WITH_CAMELLIA_128_CBC_SHA256",
-    "TLS_ECDHE_RSA_WITH_CAMELLIA_256_CBC_SHA384",
-    "TLS_RSA_WITH_AES_128_CCM",
-    "TLS_RSA_WITH_AES_256_CCM",
-    "TLS_DHE_RSA_WITH_AES_128_CCM",
-    "TLS_DHE_RSA_WITH_AES_256_CCM",
-    "TLS_RSA_WITH_AES_128_CCM_8",
-    "TLS_RSA_WITH_AES_256_CCM_8",
-    "TLS_DHE_RSA_WITH_AES_128_CCM_8",
-    "TLS_DHE_RSA_WITH_AES_256_CCM_8",
-    "TLS_ECDHE_ECDSA_WITH_AES_128_CCM",
-    "TLS_ECDHE_ECDSA_WITH_AES_256_CCM",
-    "TLS_ECDHE_ECDSA_WITH_AES_128_CCM_8",
-    "TLS_ECDHE_ECDSA_WITH_AES_256_CCM_8",
-    "TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256",
-    "TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256",
-    "TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256",
-]
-
-tls13_cs = [
-    "TLS_AES_128_GCM_SHA256",
-    "TLS_AES_256_GCM_SHA384",
-    "TLS_CHACHA20_POLY1305_SHA256",
 ]
 
 
@@ -82,15 +100,15 @@ class TestCase(TlsSuiteTester):
     For more information refer to the documentation of the TcRecorder class.
     """
 
-    sp_out_yaml = "profile_basic_openssl3_0_0"
-    recorder_yaml = "recorder_eval_cipher_suites_openssl3_0_0"
+    sp_out_yaml = "profile_basic_openssl1_0_2"
+    recorder_yaml = "recorder_eval_cipher_suites_openssl1_0_2"
     path = pathlib.Path(__file__)
     server_cmd = (
         "utils/start_openssl --version {library} --port {server_port} "
         "--cert1 server-rsa --cert2 server-ecdsa "
         "-- -www -cipher ALL"
     )
-    library = TlsLibrary.openssl3_0_0
+    library = TlsLibrary.openssl1_0_2
 
     server = "localhost"
 
@@ -107,24 +125,31 @@ class TestCase(TlsSuiteTester):
         assert versions[0]["support"] == "FALSE"
 
         assert versions[1]["version"]["name"] == "SSL30"
-        assert versions[1]["support"] == "FALSE"
+        assert versions[1]["support"] == "TRUE"
+        assert versions[1]["ciphers"]["server_preference"] == "FALSE"
 
         assert versions[2]["version"]["name"] == "TLS10"
-        assert versions[2]["support"] == "FALSE"
+        assert versions[2]["support"] == "TRUE"
+        assert versions[2]["ciphers"]["server_preference"] == "FALSE"
 
         assert versions[3]["version"]["name"] == "TLS11"
-        assert versions[3]["support"] == "FALSE"
+        assert versions[3]["support"] == "TRUE"
+        assert versions[3]["ciphers"]["server_preference"] == "FALSE"
 
         assert versions[4]["version"]["name"] == "TLS12"
         assert versions[4]["support"] == "TRUE"
         assert versions[4]["ciphers"]["server_preference"] == "FALSE"
 
         assert versions[5]["version"]["name"] == "TLS13"
-        assert versions[5]["support"] == "TRUE"
-        assert versions[5]["ciphers"]["server_preference"] == "FALSE"
-        for a, b in zip(tls12_cs, versions[4]["ciphers"]["cipher_suites"]):
+        assert versions[5]["support"] == "FALSE"
+
+        for a, b in zip(tls10_cs, versions[1]["ciphers"]["cipher_suites"]):
             assert a == b["name"]
-        for a, b in zip(tls13_cs, versions[5]["ciphers"]["cipher_suites"]):
+        for a, b in zip(tls10_cs, versions[2]["ciphers"]["cipher_suites"]):
+            assert a == b["name"]
+        for a, b in zip(tls10_cs, versions[3]["ciphers"]["cipher_suites"]):
+            assert a == b["name"]
+        for a, b in zip(tls12_cs, versions[4]["ciphers"]["cipher_suites"]):
             assert a == b["name"]
 
     def check_profile(self, profile):
@@ -135,8 +160,6 @@ class TestCase(TlsSuiteTester):
         for vers in ["sslv2", "sslv3", "tls10", "tls11", "tls12", "tls13"]:
             tlsmate.config.set(vers, True)
         server_profile = tlsmate.server_profile
-        client = tlsmate.client
-        client.init_profile()
         ScanCipherSuites(tlsmate).run()
 
         self.check_profile(server_profile.make_serializable())
