@@ -5,25 +5,30 @@
 import os
 import logging
 from pathlib import Path
+from typing import Dict, Any, Optional, ItemsView, Union
 
 # import own stuff
-from tlsmate.structs import ConfigItem
+import tlsmate.structs as structs
 
 # import other stuff
 import configparser
 
-config_logging = ConfigItem("logging", type=str, default="error")
-config_host = ConfigItem("host", type=str, default="localhost")
-config_port = ConfigItem("port", type=int, default=443)
-config_interval = ConfigItem("interval", type=int, default=0)
-config_key_log_file = ConfigItem("key_log_file", type=str, default=None)
-config_progress = ConfigItem("progress", type=bool, default=False)
-config_sni = ConfigItem("sni", type=str, default=None)
-config_ca_certs = ConfigItem("ca_certs", type="file_list")
-config_client_key = ConfigItem("client_key", type="file_list")
-config_client_chain = ConfigItem("client_chain", type="file_list")
-config_crl = ConfigItem("crl", type=bool, default=True)
-config_ocsp = ConfigItem("ocsp", type=bool, default=True)
+config_logging = structs.ConfigItem("logging", type=str, default="error")
+config_host = structs.ConfigItem("host", type=str, default="localhost")
+config_port = structs.ConfigItem("port", type=int, default=443)
+config_interval = structs.ConfigItem("interval", type=int, default=0)
+config_key_log_file = structs.ConfigItem("key_log_file", type=str, default=None)
+config_progress = structs.ConfigItem("progress", type=bool, default=False)
+config_sni = structs.ConfigItem("sni", type=str, default=None)
+config_ca_certs = structs.ConfigItem("ca_certs", type="file_list")
+config_client_key = structs.ConfigItem("client_key", type="file_list")
+config_client_chain = structs.ConfigItem("client_chain", type="file_list")
+config_crl = structs.ConfigItem("crl", type=bool, default=True)
+config_ocsp = structs.ConfigItem("ocsp", type=bool, default=True)
+config_plugin = structs.ConfigItem("plugin", type="str_list", default=None)
+config_recorder_delay = structs.ConfigItem("recorder_delay", type=bool, default=True)
+config_proxy = structs.ConfigItem("proxy", type=str, default=None)
+config_ipv6_preference = structs.ConfigItem("ipv6_preference", type=bool, default=False)
 
 
 class Configuration(object):
@@ -57,9 +62,11 @@ class Configuration(object):
             logging = debug
     """
 
-    def __init__(self):
-        self._config = {}
-        self._descr = {}
+    def __init__(self) -> None:
+        self._config_section: Optional[configparser.SectionProxy] = None
+        self._config_section_read = False
+        self._config: Dict[str, Any] = {}
+        self._descr: Dict[str, structs.ConfigItem] = {}
 
         # register configurations which are essential in the core
         # part of tlsmate.
@@ -76,14 +83,12 @@ class Configuration(object):
             config_client_chain,
             config_crl,
             config_ocsp,
+            config_plugin,
+            config_recorder_delay,
+            config_proxy,
         ]:
             self.register(item)
 
-        # special configuration item exclusively used for unit tests.
-        # if set to False, the recorder will not use any delays when replaying.
-        self.register(ConfigItem("recorder_delay", type=bool, default=True))
-        self._init_environment_var("recorder_delay")
-
     def _str_to_filelist(self, string):
         """Resolves a string of files paths.
 
@@ -111,61 +116,53 @@ class Configuration(object):
         bool: lambda self, x: x.lower() not in ["0", "off", "no", "false"],
         int: lambda self, x: int(x),
         "file_list": _str_to_filelist,
+        "str_list": lambda self, x: [token.strip() for token in x.split(",")],
     }
 
-    def _init_from_ini_file(self, ini_file):
-        """Helper method to initialize the configuration from an ini-file.
-        """
+    def _init_config_section(self, ini_file: Optional[Union[str, Path]]) -> None:
+        if self._config_section_read:
+            return
 
+        self._config_section_read = True
         if ini_file is None:
-            ini_file = Path.home() / ".tlsmate.ini"
-            if not ini_file.is_file():
+            file_path = Path.home() / ".tlsmate.ini"
+            if not file_path.is_file():
                 return
 
         else:
-            ini_file = Path(ini_file)
-            if not ini_file.is_absolute():
-                ini_file = Path.cwd() / ini_file
+            file_path = Path(ini_file)
+            if not file_path.is_absolute():
+                file_path = Path.cwd() / ini_file
 
-        if not ini_file.is_file():
-            raise FileNotFoundError(ini_file)
+            if not file_path.is_file():
+                raise FileNotFoundError(ini_file)
 
         logging.debug(f"using config file {str(ini_file)}")
-        self._config_dir = ini_file.parent
+        self._config_dir = file_path.parent
         parser = configparser.ConfigParser()
-        parser.read(str(ini_file))
+        parser.read(str(file_path))
         if parser.has_section("tlsmate"):
-            config = parser["tlsmate"]
-            for item in self._config:
-                val = config.get(item)
-                if val is not None:
-                    self._config[item] = self._cast_item(item, val)
+            self._config_section = parser["tlsmate"]
 
-    def _init_environment_var(self, item):
-        """Helper method to initialize a single item from the environment variable.
-        """
-
-        val = os.environ.get("TLSMATE_" + item.upper())
-        if val is not None:
-            self._config[item] = self._cast_item(item, val)
-
-    def _init_from_environment(self):
-        """Helper method to initialize the configuration from environment variables.
-        """
-
-        for item in self._config:
-            self._init_environment_var(item)
-
-    def _cast_item(self, item, val):
-        """Cast the type of a configuration item into the internal format.
-        """
+    def _cast_item(self, item: str, val: Any) -> Any:
+        """Cast the type of a configuration item into the internal format."""
 
         item_type = self._descr[item].type
         if item_type in self._format_option:
             val = self._format_option[item_type](self, val)
         return val
 
-    def init_from_external(self, ini_file=None):
+    def _determine_value(self, item: str) -> Any:
+        val = os.environ.get("TLSMATE_" + item.upper())
+        if val is None and self._config_section:
+            val = self._config_section.get(item)  # type: ignore
+
+        if val is not None:
+            val = self._cast_item(item, val)
+
+        return val
+
+    def init_from_external(self, ini_file: Optional[Union[str, Path]] = None):
         """Take the configuration from the ini file and from the environment variables.
 
         Arguments:
@@ -173,42 +170,51 @@ class Configuration(object):
                 the environment variables are taken into account.
         """
 
-        self._init_from_ini_file(ini_file)
-        self._init_from_environment()
+        self._init_config_section(ini_file)
+        for item in self._config:
+            val = self._determine_value(item)
+            if val is not None:
+                self._config[item] = val
 
-    def items(self):
+    def get_from_external(self, ini_file: Optional[str], item: str) -> Any:
+        """Gets the value for a single itemerving the ini-file and the environment."""
+
+        self._init_config_section(ini_file)
+        return self._determine_value(item)
+
+    def items(self) -> ItemsView[str, structs.ConfigItem]:
         """Return the configuration items. Mimics the dict's `items` method.
 
         Returns:
-            list (tuple): The list of configuration items. Each item is a tuple of the
-            item name and the item value.
+            The list of configuration items. Each item is a tuple of the item
+            name and the item value.
 
         """
 
         return self._config.items()
 
-    def get(self, key, default=None):
+    def get(self, key: str, default: Any = None) -> Any:
         """Get the value of a configuration item. Mimics the dict's `get` method.
 
         Arguments:
-            key (str): the name of the configuration item
+            key: the name of the configuration item
             default: the default value to return in case the configuration item is
                 not existing. Defaults to None.
 
         Returns:
-            any: the value of the configuration item or the provided default value if
+            the value of the configuration item or the provided default value if
             it is not present.
         """
 
         return self._config.get(key, default)
 
-    def set(self, key, val, keep_existing=True):
+    def set(self, key: str, val: Any, keep_existing: bool = True) -> None:
         """Add a configuration option.
 
         Arguments:
-            key (str): the name of the option
+            key: the name of the option
             val: the value of the option
-            keep_existing (bool): if set to True and the value is None, an existing
+            keep_existing: if set to True and the value is None, an existing
                 configuration will not be overwritten. Defaults to True
         """
 
@@ -217,12 +223,11 @@ class Configuration(object):
 
         self._config[key] = val
 
-    def register(self, config_item):
+    def register(self, config_item: structs.ConfigItem) -> None:
         """Add a new item to the configuration
 
         Arguments:
-            config_item (:obj:`tlsmate.structs.ConfigItem`): the configuration
-                item to register
+            config_item: the configuration item to register
 
         Raises:
             ValueError: if a configuration item with the same name is already existing
