@@ -5,12 +5,14 @@
 # import basic stuff
 import abc
 import time
+import logging
+from typing import Tuple, Any, Optional, List, Union
 
 # import own stuff
-from tlsmate.exception import ServerMalfunction
-from tlsmate import tls
-from tlsmate import structs
-from tlsmate import pdu
+import tlsmate.client_state as client_state
+import tlsmate.tls as tls
+import tlsmate.structs as structs
+import tlsmate.pdu as pdu
 
 # import other stuff
 
@@ -19,33 +21,38 @@ class Extension(metaclass=abc.ABCMeta):
     """Abstract class for all extensions.
     """
 
+    extension_id: tls.Extension
+    """The extension id
+    """
+
     @abc.abstractmethod
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session: client_state.SessionState) -> bytes:
         """Serializes the content of an extension, i.e. excluding the header.
 
         Arguments:
-            conn (:obj:`tlsmate.connection.TlsConnection`): the connection object
-                that will be used to send this extensions.
+            session: the session object that will be used to send this
+                extensions.
 
         Returns:
             bytes: The serialized extensions body.
         """
+
         pass
 
-    def serialize(self, conn):
+    def serialize(self, session: client_state.SessionState) -> bytes:
         """Serializes an extensions, including the extensions header.
 
         Arguments:
-            conn (:obj:`tlsmate.connection.TlsConnection`): The connection object,
-                needed for some odd cases, e.g. when serializing a key share
+            session: the session object that will be used to send this
                 extensions.
 
         Returns:
-            bytes: The serialized extension.
+            The serialized extension.
         """
-        ext_body = self._serialize_ext_body(conn)
+
+        ext_body = self._serialize_ext_body(session)
         if self.extension_id is tls.Extension.UNKNOW_EXTENSION:
-            ext_id = self.id
+            ext_id = self.id  # type: ignore
 
         else:
             ext_id = self.extension_id.value
@@ -56,16 +63,16 @@ class Extension(metaclass=abc.ABCMeta):
         return ext
 
     @staticmethod
-    def deserialize(fragment, offset):
+    def deserialize(fragment: bytes, offset: int) -> Tuple["Extension", int]:
         """Deserializes an extension.
 
         Arguments:
-            fragment (bytes): A PDU buffer as received from the network.
-            offset (int): The offset within the fragment where the extension starts.
+            fragment: A PDU buffer as received from the network.
+            offset: The offset within the fragment where the extension starts.
 
         Returns:
-            tuple (:obj:`Extension`, new offset): The deserialized extension as an
-            python object and the new offset into the fragment.
+            The deserialized extension as an python object and the new offset
+            into the fragment.
         """
         ext_id_int, offset = pdu.unpack_uint16(fragment, offset)
         ext_id = tls.Extension.val2enum(ext_id_int)
@@ -92,17 +99,17 @@ class ExtUnknownExtension(Extension):
     """Any extensions which is not known by tlsmate (yet).
 
     Arguments:
-        id (int): the Id of the extensions as used in the PDU
-        bytes (bytes): the content of the extension as a byte string
+        id: the Id of the extensions as used in the PDU
+        bytes: the content of the extension as a byte string
     """
 
     extension_id = tls.Extension.UNKNOW_EXTENSION
 
-    def __init__(self, **kwargs):
-        self.id = kwargs.get("id")
-        self.bytes = kwargs.get("bytes")
+    def __init__(self, **kwargs: Any) -> None:
+        self.id: Optional[int] = kwargs.get("id")
+        self.bytes: Optional[bytes] = kwargs.get("bytes")
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         return self.bytes
 
     def _deserialize_ext_body(self, fragment):
@@ -114,17 +121,17 @@ class ExtServerNameIndication(Extension):
     """Represents the ServerNameIndication extension.
 
     Attributes:
-        host_name (str): The name which identifies the host.
+        host_name: The name which identifies the host.
     """
 
     extension_id = tls.Extension.SERVER_NAME
     """:obj:`tlsmate.tls.Extension.SERVER_NAME`
     """
 
-    def __init__(self, host_name=None):
+    def __init__(self, host_name: Optional[str] = None) -> None:
         self.host_name = host_name
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         # we only support exacly one list element: host_name
         ext = bytearray(pdu.pack_uint8(0))  # host_name
         ext.extend(pdu.pack_uint16(len(self.host_name)))
@@ -139,7 +146,7 @@ class ExtServerNameIndication(Extension):
 
         list_length, offset = pdu.unpack_uint16(fragment, 0)
         if offset + list_length != len(fragment):
-            raise ServerMalfunction(
+            raise tls.ServerMalfunction(
                 tls.ServerIssue.EXTENTION_LENGHT_ERROR, extension=self.extension_id
             )
 
@@ -151,7 +158,7 @@ class ExtServerNameIndication(Extension):
                 self.host_name = name.decode()
 
         if self.host_name is None:
-            raise ServerMalfunction(
+            raise tls.ServerMalfunction(
                 tls.ServerIssue.SNI_NO_HOSTNAME, extension=self.extension_id
             )
 
@@ -164,12 +171,12 @@ class ExtExtendedMasterSecret(Extension):
     """:obj:`tlsmate.tls.Extension.EXTENDED_MASTER_SECRET`
     """
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         return b""
 
     def _deserialize_ext_body(self, ext_body):
         if ext_body:
-            raise ServerMalfunction(
+            raise tls.ServerMalfunction(
                 tls.ServerIssue.EXTENTION_LENGHT_ERROR, extension=self.extension_id
             )
 
@@ -182,12 +189,12 @@ class ExtEncryptThenMac(Extension):
     """:obj:`tlsmate.tls.Extension.ENCRYPT_THEN_MAC`
     """
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         return b""
 
     def _deserialize_ext_body(self, ext_body):
         if ext_body:
-            raise ServerMalfunction(
+            raise tls.ServerMalfunction(
                 tls.ServerIssue.EXTENTION_LENGHT_ERROR, extension=self.extension_id
             )
 
@@ -196,17 +203,17 @@ class ExtRenegotiationInfo(Extension):
     """Represents the RenegotiationInfo extension.
 
     Attributes:
-        opaque (bytes): The opaque bytes.
+        renegotiated_connection: The opaque bytes.
     """
 
     extension_id = tls.Extension.RENEGOTIATION_INFO
     """:obj:`tlsmate.tls.Extension.RENEGOTIATION_INFO`
     """
 
-    def __init__(self, renegotiated_connection=b"\0"):
+    def __init__(self, renegotiated_connection: bytes = b"\0") -> None:
         self.renegotiated_connection = renegotiated_connection
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         ext_body = (
             pdu.pack_uint8(len(self.renegotiated_connection))
             + self.renegotiated_connection
@@ -224,20 +231,21 @@ class ExtEcPointFormats(Extension):
     """Represents the EcPointFormat extension.
 
     Attributes:
-        ec_point_formats (list of :obj:`tlsmate.tls.EcPointFormat`): The list
-            of supported point formats.
+        ec_point_formats: The list of supported point formats.
     """
 
     extension_id = tls.Extension.EC_POINT_FORMATS
     """:obj:`tlsmate.tls.Extension.EC_POINT_FORMATS`
     """
 
-    def __init__(self, ec_point_formats=None):
+    def __init__(
+        self, ec_point_formats: Optional[List[Union[tls.EcPointFormat, int]]] = None
+    ) -> None:
         self.ec_point_formats = (
             ec_point_formats if ec_point_formats else [tls.EcPointFormat.UNCOMPRESSED]
         )
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         format_list = bytearray()
         for point_format in self.ec_point_formats:
             if type(point_format) == int:
@@ -255,7 +263,7 @@ class ExtEcPointFormats(Extension):
         self.ec_point_formats = []
         length, offset = pdu.unpack_uint8(ext_body, 0)
         if offset + length != len(ext_body):
-            raise ServerMalfunction(
+            raise tls.ServerMalfunction(
                 tls.ServerIssue.EXTENTION_LENGHT_ERROR, extension=self.extension_id
             )
 
@@ -268,18 +276,19 @@ class ExtSupportedGroups(Extension):
     """Represents the SupportedGroup extension.
 
     Attributes:
-        supported_groups (list of :obj:`tlsmate.tls.SupportedGroups`): The list
-            of supported groups.
+        supported_groups: The list of supported groups.
     """
 
     extension_id = tls.Extension.SUPPORTED_GROUPS
     """:obj:`tlsmate.tls.Extension.SUPPORTED_GROUPS`
     """
 
-    def __init__(self, supported_groups=None):
+    def __init__(
+        self, supported_groups: Optional[List[tls.SupportedGroups]] = None
+    ) -> None:
         self.supported_groups = supported_groups if supported_groups else []
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         group_list = bytearray()
         for group in self.supported_groups:
             group_list.extend(pdu.pack_uint16(getattr(group, "value", group)))
@@ -307,18 +316,20 @@ class ExtSignatureAlgorithms(Extension):
     """Represents the SignatureAlgorithms extension.
 
     Attributes:
-        signature_algorithms (list of :obj:`tlsmate.tls.SignatureScheme`): The
-            of supported signature algorithms.
+        signature_algorithms: The of supported signature algorithms.
     """
 
     extension_id = tls.Extension.SIGNATURE_ALGORITHMS
     """:obj:`tlsmate.tls.Extension.SIGNATURE_ALGORITHMS`
     """
 
-    def __init__(self, signature_algorithms=None):
+    def __init__(
+        self,
+        signature_algorithms: Optional[List[Union[tls.SignatureScheme, int]]] = None,
+    ) -> None:
         self.signature_algorithms = signature_algorithms if signature_algorithms else []
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         algo_list = bytearray()
         for algo in self.signature_algorithms:
             if type(algo) is tuple:
@@ -348,18 +359,17 @@ class ExtHeartbeat(Extension):
     """Represents the Heartbeat extension.
 
     Attributes:
-        heartbeat_mode (:obj:`tlsmate.tls.HeartbeatMode`): The mode for the
-            heartbeat extension.
+        heartbeat_mode: The mode for the heartbeat extension.
     """
 
     extension_id = tls.Extension.HEARTBEAT
     """:obj:`tlsmate.tls.Extension.HEARTBEAT`
     """
 
-    def __init__(self, heartbeat_mode=None):
+    def __init__(self, heartbeat_mode: Optional[tls.HeartbeatMode] = None) -> None:
         self.heartbeat_mode = heartbeat_mode
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         if type(self.heartbeat_mode) is int:
             val = self.heartbeat_mode
         else:
@@ -376,17 +386,17 @@ class ExtCertificateAuthorities(Extension):
     """Represents the CertificateAuthorities extension.
 
     Attributes:
-        authorities (list of bytes): The list of authorities in original ASN.1 format.
+        authorities: The list of authorities in original ASN.1 format.
     """
 
     extension_id = tls.Extension.CERTIFICATE_AUTHORITIES
     """:obj:`tlsmate.tls.Extension.CERTIFICATE_AUTHORITIES`
     """
 
-    def __init__(self, authorities=None):
+    def __init__(self, authorities: Optional[List[bytes]] = None) -> None:
         self.authorities = authorities if authorities else []
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         raise NotImplementedError(f"serialization of extension {self} not implemented")
 
     def _deserialize_ext_body(self, ext_body):
@@ -402,17 +412,17 @@ class ExtSessionTicket(Extension):
     """Represents the SessionTicket extension.
 
     Attributes:
-        ticket (bytes): The ticket.
+        ticket: The ticket.
     """
 
     extension_id = tls.Extension.SESSION_TICKET
     """:obj:`tlsmate.tls.Extension.SESSION_TICKET`
     """
 
-    def __init__(self, ticket=None):
+    def __init__(self, ticket: Optional[bytes] = None) -> None:
         self.ticket = ticket
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         ext_body = bytearray()
         if self.ticket is not None:
             ext_body.extend(self.ticket)
@@ -427,12 +437,10 @@ class ExtStatusRequest(Extension):
     """Represents the status_request extension.
 
     Attributes:
-        status_type (:obj:`tlsmate.tls.StatusType`): The status type. Defaults to
+        status_type: The status type. Defaults to
             :obj:`tlsmate.tls.StatusType.OCSP`.
-        responder_ids (list of bytes): The list of responder ids. Defaults to an empty
-            list.
-        extensions (list of bytes): The list of extensions. Defaults to an empty
-            list.
+        responder_ids: The list of responder ids. Defaults to an empty list.
+        extensions: The list of extensions. Defaults to an empty list.
     """
 
     extension_id = tls.Extension.STATUS_REQUEST
@@ -440,8 +448,11 @@ class ExtStatusRequest(Extension):
     """
 
     def __init__(
-        self, status_type=tls.StatusType.OCSP, responder_ids=None, extensions=None
-    ):
+        self,
+        status_type: tls.StatusType = tls.StatusType.OCSP,
+        responder_ids: Optional[List[bytes]] = None,
+        extensions: Optional[List[bytes]] = None,
+    ) -> None:
         self.status_type = status_type
         if responder_ids is None:
             responder_ids = []
@@ -453,7 +464,7 @@ class ExtStatusRequest(Extension):
         self.extensions = extensions
         self.ocsp_response = None
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         ext_body = bytearray()
         ext_body.extend(
             pdu.pack_uint8(getattr(self.status_type, "value", self.status_type))
@@ -504,9 +515,12 @@ class ExtStatusRequestV2(Extension):
     """
 
     def __init__(
-        self, status_type=tls.StatusType.OCSP_MULTI, responder_ids=None, extensions=b""
-    ):
-        self._requests = []
+        self,
+        status_type: tls.StatusType = tls.StatusType.OCSP_MULTI,
+        responder_ids: Optional[List[bytes]] = None,
+        extensions: bytes = b"",
+    ) -> None:
+        self._requests: List[Tuple[tls.StatusType, Optional[List[bytes]], bytes]] = []
         self.add_request(status_type, responder_ids, extensions)
 
     def add_request(self, status_type, responder_ids=None, extensions=b""):
@@ -515,7 +529,7 @@ class ExtStatusRequestV2(Extension):
 
         self._requests.append((status_type, responder_ids, extensions))
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
 
         ext_body = bytearray()
         for status_type, responder_ids, extensions in self._requests:
@@ -553,18 +567,19 @@ class ExtSupportedVersions(Extension):
     """Represents the SupportedVersion extension.
 
     Attributes:
-        versions (list of :obj:`tlsmate.tls.Version`): The list of TLS versions
-            supported.
+        versions: The list of TLS versions supported.
     """
 
     extension_id = tls.Extension.SUPPORTED_VERSIONS
     """:obj:`tlsmate.tls.Extension.SUPPORTED_VERSIONS`
     """
 
-    def __init__(self, versions=None):
+    def __init__(
+        self, versions: Optional[List[Union[tls.Version, int]]] = None
+    ) -> None:
         self.versions = versions
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         versions = bytearray()
         for version in self.versions:
             versions.extend(pdu.pack_uint16(getattr(version, "value", version)))
@@ -583,6 +598,31 @@ class ExtSupportedVersions(Extension):
             self.versions.append(version)
 
 
+class ExtCookie(Extension):
+    """Represents the Cookie extension.
+
+    Attributes:
+        cookie: The opaque cookie value
+    """
+
+    extension_id = tls.Extension.COOKIE
+    """:obj:`tlsmate.tls.Extension.COOKIE`
+    """
+
+    def __init__(self, cookie: bytes = b"") -> None:
+        self.cookie = cookie
+
+    def _serialize_ext_body(self, session):
+        ext_body = bytearray()
+        ext_body.extend(pdu.pack_uint16(len(self.cookie)))
+        ext_body.extend(self.cookie)
+        return ext_body
+
+    def _deserialize_ext_body(self, ext_body):
+        length, offset = pdu.unpack_uint16(ext_body, 0)
+        self.cookie, offset = pdu.unpack_bytes(ext_body, offset, length)
+
+
 class ExtKeyShare(Extension):
     """Represents the KeyShare extension.
 
@@ -592,22 +632,42 @@ class ExtKeyShare(Extension):
     provided in the SupportedGroup extension.
 
     Attributes:
-        key_shares (list of :obj:`tlsmate.tls.SupportedGroups`): The list of
-            supported groups for which key shares shall be generated.
+        key_shares (list of :obj:`tlsmate.structs.KeyShareEntry`): the list of
+            key shares, comprising the group and the public key for each element.
+        groups (list of :obj:`tlsmate.tls.SupportedGroups`): The list of
+            supported groups for which key shares shall be generated. This is an
+            alternative to setup the extension. The public key will be determined
+            when serializing the extension.
     """
 
     extension_id = tls.Extension.KEY_SHARE
     """:obj:`tlsmate.tls.Extension.KEY_SHARE`
     """
 
-    def __init__(self, key_shares=None):
+    def __init__(
+        self,
+        key_shares: Optional[List[structs.KeyShareEntry]] = None,
+        groups: Optional[List[tls.SupportedGroups]] = None,
+    ) -> None:
+        if key_shares is None:
+            key_shares = []
+
+        if groups is None:
+            groups = []
+
+        if not key_shares and groups:
+            key_shares = [
+                structs.KeyShareEntry(group=group, key_exchange=None)
+                for group in groups
+            ]
+
         self.key_shares = key_shares
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         key_shares = bytearray()
-        for group in self.key_shares:
-            key_shares.extend(pdu.pack_uint16(group.value))
-            share = conn.get_key_share(group)
+        for ks in self.key_shares:
+            key_shares.extend(pdu.pack_uint16(ks.group.value))
+            share = session.create_key_share(ks.group)
             key_shares.extend(pdu.pack_uint16(len(share)))
             key_shares.extend(share)
 
@@ -622,8 +682,13 @@ class ExtKeyShare(Extension):
         while offset < len(ext_body):
             group, offset = pdu.unpack_uint16(ext_body, offset)
             group = tls.SupportedGroups.val2enum(group, alert_on_failure=True)
-            length, offset = pdu.unpack_uint16(ext_body, offset)
-            share, offset = pdu.unpack_bytes(ext_body, offset, length)
+            if offset < len(ext_body):
+                length, offset = pdu.unpack_uint16(ext_body, offset)
+                share, offset = pdu.unpack_bytes(ext_body, offset, length)
+
+            else:
+                share = None
+
             self.key_shares.append(
                 structs.KeyShareEntry(group=group, key_exchange=share)
             )
@@ -635,25 +700,24 @@ class ExtPreSharedKey(Extension):
     Builds this extension based on the list of the given pre shared keys.
 
     Attributes:
-        psks (list of :obj:`tlsmate.structs.Psk`): The list of
-            pre shared keys to offer to the server.
+        psks: The list of pre shared keys to offer to the server.
     """
 
     extension_id = tls.Extension.PRE_SHARED_KEY
     """:obj:`tlsmate.tls.Extension.PRE_SHARED_KEY`
     """
 
-    def __init__(self, psks=None):
+    def __init__(self, psks: Optional[List[structs.Psk]] = None) -> None:
         self.psks = psks
         self._bytes_after_ids = 0
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         identities = bytearray()
         binders = bytearray()
         for psk in self.psks:
             identities.extend(pdu.pack_uint16(len(psk.ticket)))
             identities.extend(psk.ticket)
-            timestamp = conn.recorder.inject(timestamp=time.time())
+            timestamp = session.recorder.inject(timestamp=time.time())
             ticket_age = int((timestamp - psk.timestamp) * 1000 + psk.age_add) % (
                 2 ** 32
             )
@@ -679,18 +743,17 @@ class ExtPskKeyExchangeMode(Extension):
     """Represents the psk_key_exchange_mode extension.
 
     Attributes:
-        modes (list of :obj:`tlsmate.tls.PskKeyExchangeMode`): The list of
-            the PSK key exchange modes to offer to the server.
+        modes: The list of the PSK key exchange modes to offer to the server.
     """
 
     extension_id = tls.Extension.PSK_KEY_EXCHANGE_MODES
     """:obj:`tlsmate.tls.Extension.PSK_KEY_EXCHANGE_MODES`
     """
 
-    def __init__(self, modes=None):
+    def __init__(self, modes: Optional[List[tls.PskKeyExchangeMode]] = None) -> None:
         self.modes = modes
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         ext_body = bytearray(pdu.pack_uint8(len(self.modes)))
         for mode in self.modes:
             ext_body.extend(pdu.pack_uint8(getattr(mode, "value", mode)))
@@ -703,16 +766,19 @@ class ExtPskKeyExchangeMode(Extension):
 
 class ExtEarlyData(Extension):
     """Represents the EarlyData extension.
+
+    Attributes:
+        max_early_data_size: The maximum number of bytes used for the early data.
     """
 
     extension_id = tls.Extension.EARLY_DATA
     """:obj:`tlsmate.tls.Extension.EARLY_DATA`
     """
 
-    def __init__(self, max_early_data_size=None):
+    def __init__(self, max_early_data_size: Optional[int] = None) -> None:
         self.max_early_data_size = max_early_data_size
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         if self.max_early_data_size is None:
             return b""
 
@@ -732,12 +798,12 @@ class ExtPostHandshakeAuth(Extension):
     """:obj:`tlsmate.tls.Extension.POST_HANDSHAKE_AUTH`
     """
 
-    def _serialize_ext_body(self, conn):
+    def _serialize_ext_body(self, session):
         return b""
 
     def _deserialize_ext_body(self, ext_body):
         if ext_body:
-            raise ServerMalfunction(
+            raise tls.ServerMalfunction(
                 tls.ServerIssue.EXTENTION_LENGHT_ERROR, extension=self.extension_id
             )
 
@@ -784,7 +850,7 @@ deserialization_map = {
     tls.Extension.PRE_SHARED_KEY: ExtPreSharedKey,
     tls.Extension.EARLY_DATA: ExtEarlyData,
     tls.Extension.SUPPORTED_VERSIONS: ExtSupportedVersions,
-    # tls.Extension.COOKIE = 44
+    tls.Extension.COOKIE: ExtCookie,
     # tls.Extension.PSK_KEY_EXCHANGE_MODES = 45
     tls.Extension.CERTIFICATE_AUTHORITIES: ExtCertificateAuthorities,
     # tls.Extension.OID_FILTERS = 48
@@ -797,3 +863,61 @@ deserialization_map = {
     # tls.Extension.EXTERNAL_SESSION_ID = 56
     tls.Extension.RENEGOTIATION_INFO: ExtRenegotiationInfo,
 }
+
+
+def log_extensions(extensions: List[Extension]) -> None:
+    """Log extensions
+
+    Arguments:
+        extensions: the list of extensions to iterate over
+    """
+
+    for extension in extensions:
+        ext_id = extension.extension_id
+        logging.debug(f"extension {ext_id.value} {ext_id}")
+
+
+def get_extension(
+    extensions: List[Extension], ext_id: tls.Extension
+) -> Optional[Extension]:
+    """Helper function to search for an extension
+
+    Arguments:
+        extensions: the list to search for the extension
+        ext_id: the extension to to look for
+
+    Returns:
+        The extension if present, or None otherwise.
+    """
+
+    if extensions is not None:
+        for extension in extensions:
+            if extension.extension_id == ext_id:
+                return extension
+
+    return None
+
+
+def deserialize_extensions(
+    extensions: List[Extension], fragment: bytes, offset: int
+) -> int:
+    """Helper function to deserialize extensions
+
+    Arguments:
+        extensions: the list where to store the deserialized extensions
+        fragment: the pdu buffer
+        offset: the offset within the pdu buffer to the start of the extensions
+
+    Returns:
+        the offset on the byte following the extensions
+    """
+
+    if offset < len(fragment):
+        # extensions present
+        ext_len, offset = pdu.unpack_uint16(fragment, offset)
+        ext_end = offset + ext_len
+        while offset < ext_end:
+            extension, offset = Extension.deserialize(fragment, offset)
+            extensions.append(extension)
+
+    return offset
