@@ -3,54 +3,44 @@
 """
 # import basic stuff
 from pathlib import Path
+import argparse
+from typing import Any
 
 # import own stuff
-from tlsmate import utils
-from tlsmate.structs import ConfigItem
-from tlsmate.plugin import BaseCommand, Plugin, Args, WorkManager
-from tlsmate.workers.compression import ScanCompression
-from tlsmate.workers.encrypt_then_mac import ScanEncryptThenMac
-from tlsmate.workers.master_secret import ScanExtendedMasterSecret
-from tlsmate.workers.resumption import ScanResumption
-from tlsmate.workers.renegotiation import ScanRenegotiation
-from tlsmate.workers.ccs_injection import ScanCcsInjection
-from tlsmate.workers.robot import ScanRobot
-from tlsmate.workers.padding_oracle import ScanPaddingOracle
-from tlsmate.workers.dh_params import ScanDhGroups
-from tlsmate.workers.heartbeat import ScanHeartbeat
-from tlsmate.workers.heartbleed import ScanHeartbleed
-from tlsmate.workers.grease import ScanGrease
-from tlsmate.workers.ephemeral_key_reuse import ScanEphemeralKeyReuse
-from tlsmate.workers.ocsp_stapling import ScanOcspStapling
-from tlsmate.workers.downgrade import ScanDowngrade
-from tlsmate.workers.server_profile import DumpProfileWorker, ReadProfileWorker
-from tlsmate.workers.text_server_profile import TextProfileWorker
-from tlsmate.workers.eval_cipher_suites import ScanCipherSuites
-from tlsmate.workers.scanner_info import ScanStart, ScanEnd
-from tlsmate.workers.supported_groups import ScanSupportedGroups
-from tlsmate.workers.sig_algo import ScanSigAlgs
-from tlsmate.workers.base_vulnerabilities import ScanBaseVulnerabilities
-from tlsmate.config import (
-    config_host,
-    config_port,
-    config_interval,
-    config_key_log_file,
-    config_progress,
-    config_sni,
-    config_ca_certs,
-    config_client_key,
-    config_client_chain,
-    config_crl,
-    config_ocsp,
-)
+import tlsmate.config as conf
+import tlsmate.plugin as plg
+import tlsmate.structs as structs
+import tlsmate.utils as utils
+import tlsmate.workers.base_vulnerabilities as base_vulnerabilities
+import tlsmate.workers.ccs_injection as ccs_injection
+import tlsmate.workers.compression as compression
+import tlsmate.workers.dh_params as dh_params
+import tlsmate.workers.downgrade as downgrade
+import tlsmate.workers.encrypt_then_mac as encrypt_then_mac
+import tlsmate.workers.ephemeral_key_reuse as ephemeral_key_reuse
+import tlsmate.workers.eval_cipher_suites as eval_cipher_suites
+import tlsmate.workers.grease as grease
+import tlsmate.workers.heartbeat as heartbeat
+import tlsmate.workers.heartbleed as heartbleed
+import tlsmate.workers.master_secret as master_secret
+import tlsmate.workers.ocsp_stapling as ocsp_stapling
+import tlsmate.workers.padding_oracle as padding_oracle
+import tlsmate.workers.renegotiation as renegotiation
+import tlsmate.workers.resumption as resumption
+import tlsmate.workers.robot as robot
+import tlsmate.workers.scanner_info as scanner_info
+import tlsmate.workers.server_profile as server_profile
+import tlsmate.workers.sig_algo as sig_algo
+import tlsmate.workers.supported_groups as supported_groups
+import tlsmate.workers.text_server_profile as text_server_profile
 
 
-class ArgPort(Plugin):
+class ArgPort(plg.Plugin):
     """Argument for port.
     """
 
-    config = config_port
-    cli_args = Args(
+    config = conf.config_port
+    cli_args = plg.Args(
         "--port",
         default=None,
         help="the port number of the host [0-65535]. Defaults to 443.",
@@ -58,18 +48,24 @@ class ArgPort(Plugin):
     )
 
     @classmethod
-    def args_parsed(cls, args, parser, subcommand, config):
+    def args_parsed(
+        cls,
+        args: Any,
+        parser: argparse.ArgumentParser,
+        subcommand: str,
+        config: conf.Configuration,
+    ) -> None:
         if args.port is not None and (args.port < 0 or args.port > 0xFFFF):
             parser.error("port must be in the range [0-65535]")
         super().args_parsed(args, parser, subcommand, config)
 
 
-class ArgInterval(Plugin):
+class ArgInterval(plg.Plugin):
     """Argument for interval.
     """
 
-    config = config_interval
-    cli_args = Args(
+    config = conf.config_interval
+    cli_args = plg.Args(
         "--interval",
         default=0,
         help="the interval in milliseconds between two handshakes.",
@@ -77,12 +73,24 @@ class ArgInterval(Plugin):
     )
 
 
-class ArgKeyLogFile(Plugin):
+class ArgIpv6Preference(plg.Plugin):
+    """Argument for IPv6 preference.
+    """
+
+    config = conf.config_ipv6_preference
+    cli_args = plg.Args(
+        "--ipv6-preference",
+        help="prefers resolved IPv6 addresses over IPv4. Defaults to False.",
+        action=utils.BooleanOptionalAction,
+    )
+
+
+class ArgKeyLogFile(plg.Plugin):
     """Argument for key logging file.
     """
 
-    config = config_key_log_file
-    cli_args = Args(
+    config = conf.config_key_log_file
+    cli_args = plg.Args(
         "--key-log-file",
         default=None,
         help=(
@@ -92,24 +100,39 @@ class ArgKeyLogFile(Plugin):
     )
 
 
-class ArgProgress(Plugin):
+class ArgProgress(plg.Plugin):
     """Argument for progress.
     """
 
-    config = config_progress
-    cli_args = Args(
+    config = conf.config_progress
+    cli_args = plg.Args(
         "--progress",
         help="provides a progress indicator. Defaults to False.",
         action=utils.BooleanOptionalAction,
     )
 
 
-class ArgSni(Plugin):
+class ArgProxy(plg.Plugin):
+    """Argument for proxy.
+    """
+
+    config = conf.config_proxy
+    cli_args = plg.Args(
+        "--proxy",
+        type=str,
+        help=(
+            "the URL of the proxy. Must include the scheme and may include the user, "
+            "password and port, e.g.: `http://user:password@myproxy.net:3128`."
+        ),
+    )
+
+
+class ArgSni(plg.Plugin):
     """Argument for sni.
     """
 
-    config = config_sni
-    cli_args = Args(
+    config = conf.config_sni
+    cli_args = plg.Args(
         "--sni",
         type=str,
         help=(
@@ -121,12 +144,12 @@ class ArgSni(Plugin):
     )
 
 
-class ArgHost(Plugin):
+class ArgHost(plg.Plugin):
     """Argument for host.
     """
 
-    config = config_host
-    cli_args = Args(
+    config = conf.config_host
+    cli_args = plg.Args(
         "host",
         help=(
             "the target host. Can be given as a domain name or as an IPv4/IPv6 address."
@@ -135,26 +158,28 @@ class ArgHost(Plugin):
     )
 
 
-class GroupBasicScan(Plugin):
+class GroupBasicScan(plg.Plugin):
     """Group for basic arguments.
     """
 
     plugins = [
         ArgPort,
         ArgInterval,
+        ArgIpv6Preference,
         ArgKeyLogFile,
         ArgProgress,
+        ArgProxy,
         ArgSni,
         ArgHost,
     ]
 
 
-class ArgReadProfile(Plugin):
+class ArgReadProfile(plg.Plugin):
     """Argument for read-profile.
     """
 
-    config = ConfigItem("read_profile", type=str, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("read_profile", type=str, default=None)
+    cli_args = plg.Args(
         "--read-profile",
         type=str,
         help=(
@@ -162,15 +187,15 @@ class ArgReadProfile(Plugin):
             "determined automatically."
         ),
     )
-    workers = [ReadProfileWorker]
+    workers = [server_profile.ReadProfileWorker]
 
 
-class ArgWriteProfile(Plugin):
+class ArgWriteProfile(plg.Plugin):
     """Argument for write-profile.
     """
 
-    config = ConfigItem("write_profile", type=str, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("write_profile", type=str, default=None)
+    cli_args = plg.Args(
         "--write-profile",
         type=str,
         help=(
@@ -181,12 +206,12 @@ class ArgWriteProfile(Plugin):
     )
 
 
-class ArgFormat(Plugin):
+class ArgFormat(plg.Plugin):
     """Argument for format.
     """
 
-    config = ConfigItem("format", type=str, default="text")
-    cli_args = Args(
+    config = structs.ConfigItem("format", type=str, default="text")
+    cli_args = plg.Args(
         "--format",
         choices=["text", "html", "json", "yaml", "none"],
         help=('the output format of the server profile. Defaults to "text".'),
@@ -198,17 +223,17 @@ class ArgFormat(Plugin):
         super().args_parsed(args, parser, subcommand, config)
         format_type = config.get("format")
         if format_type in ["text", "html"]:
-            WorkManager.register(TextProfileWorker)
+            plg.WorkManager.register(text_server_profile.TextProfileWorker)
         elif format_type in ["json", "yaml"]:
-            WorkManager.register(DumpProfileWorker)
+            plg.WorkManager.register(server_profile.DumpProfileWorker)
 
 
-class ArgColor(Plugin):
+class ArgColor(plg.Plugin):
     """Argument for color.
     """
 
-    config = ConfigItem("color", type=bool, default=True)
-    cli_args = Args(
+    config = structs.ConfigItem("color", type=bool, default=True)
+    cli_args = plg.Args(
         "--color",
         help=(
             "use colored console output. Only used if --format=text is given. "
@@ -218,13 +243,15 @@ class ArgColor(Plugin):
     )
 
 
-class ArgStyle(Plugin):
+class ArgStyle(plg.Plugin):
     """Argument for style.
     """
 
     _DEFAULT_STYLE = Path(__file__).parent / "../styles/default.yaml"
-    config = ConfigItem("style", type=str, default=str(_DEFAULT_STYLE.resolve()))
-    cli_args = Args(
+    config = structs.ConfigItem(
+        "style", type=str, default=str(_DEFAULT_STYLE.resolve())
+    )
+    cli_args = plg.Args(
         "--style",
         type=str,
         help=(
@@ -235,11 +262,11 @@ class ArgStyle(Plugin):
     )
 
 
-class GroupServerProfile(Plugin):
+class GroupServerProfile(plg.Plugin):
     """Argument group for server profile arguments.
     """
 
-    group = Args(title="Server profile options")
+    group = plg.Args(title="Server profile options")
     plugins = [
         ArgWriteProfile,
         ArgFormat,
@@ -248,12 +275,12 @@ class GroupServerProfile(Plugin):
     ]
 
 
-class ArgCaCerts(Plugin):
+class ArgCaCerts(plg.Plugin):
     """Argument for CA certs.
     """
 
-    config = config_ca_certs
-    cli_args = Args(
+    config = conf.config_ca_certs
+    cli_args = plg.Args(
         "--ca-certs",
         nargs="*",
         type=str,
@@ -265,12 +292,12 @@ class ArgCaCerts(Plugin):
     )
 
 
-class ArgClientKey(Plugin):
+class ArgClientKey(plg.Plugin):
     """Argument for client keys.
     """
 
-    config = config_client_key
-    cli_args = Args(
+    config = conf.config_client_key
+    cli_args = plg.Args(
         "--client-key",
         type=str,
         nargs="*",
@@ -282,12 +309,12 @@ class ArgClientKey(Plugin):
     )
 
 
-class ArgClientChain(Plugin):
+class ArgClientChain(plg.Plugin):
     """Argument for client certificate chains.
     """
 
-    config = config_client_chain
-    cli_args = Args(
+    config = conf.config_client_chain
+    cli_args = plg.Args(
         "--client-chain",
         type=str,
         nargs="*",
@@ -300,12 +327,12 @@ class ArgClientChain(Plugin):
     )
 
 
-class ArgCrl(Plugin):
+class ArgCrl(plg.Plugin):
     """Argument for CRL.
     """
 
-    config = config_crl
-    cli_args = Args(
+    config = conf.config_crl
+    cli_args = plg.Args(
         "--crl",
         help=(
             "download the CRL to check for the certificate revocation status. "
@@ -315,12 +342,12 @@ class ArgCrl(Plugin):
     )
 
 
-class ArgOcsp(Plugin):
+class ArgOcsp(plg.Plugin):
     """Argument for OCSP.
     """
 
-    config = config_ocsp
-    cli_args = Args(
+    config = conf.config_ocsp
+    cli_args = plg.Args(
         "--ocsp",
         help=(
             "query the OCSP servers for checking the certificate "
@@ -330,11 +357,11 @@ class ArgOcsp(Plugin):
     )
 
 
-class GroupX509(Plugin):
+class GroupX509(plg.Plugin):
     """Argument group for X509 arguments.
     """
 
-    group = Args(title="X509 certificates options")
+    group = plg.Args(title="X509 certificates options")
     plugins = [ArgCaCerts, ArgClientKey, ArgClientChain, ArgCrl, ArgOcsp]
 
     @classmethod
@@ -351,84 +378,86 @@ class GroupX509(Plugin):
                     "be identical"
                 )
 
+        super().args_parsed(args, parser, subcommand, config)
 
-class ArgSslv2(Plugin):
+
+class ArgSslv2(plg.Plugin):
     """Argument for SSLv2.
     """
 
-    config = ConfigItem("sslv2", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("sslv2", type=bool, default=None)
+    cli_args = plg.Args(
         "--sslv2",
         help="scan for protocol version SSLv2",
         action=utils.BooleanOptionalAction,
     )
 
 
-class ArgSslv3(Plugin):
+class ArgSslv3(plg.Plugin):
     """Argument for SSLv3.
     """
 
-    config = ConfigItem("sslv3", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("sslv3", type=bool, default=None)
+    cli_args = plg.Args(
         "--sslv3",
         help="scan for protocol version SSLv3",
         action=utils.BooleanOptionalAction,
     )
 
 
-class ArgTls10(Plugin):
+class ArgTls10(plg.Plugin):
     """Argument for TLS1.0.
     """
 
-    config = ConfigItem("tls10", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("tls10", type=bool, default=None)
+    cli_args = plg.Args(
         "--tls10",
         help="scan for protocol version TLS1.0",
         action=utils.BooleanOptionalAction,
     )
 
 
-class ArgTls11(Plugin):
+class ArgTls11(plg.Plugin):
     """Argument for TLS1.1.
     """
 
-    config = ConfigItem("tls11", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("tls11", type=bool, default=None)
+    cli_args = plg.Args(
         "--tls11",
         help="scan for protocol version TLS1.1",
         action=utils.BooleanOptionalAction,
     )
 
 
-class ArgTls12(Plugin):
+class ArgTls12(plg.Plugin):
     """Argument for TLS1.2.
     """
 
-    config = ConfigItem("tls12", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("tls12", type=bool, default=None)
+    cli_args = plg.Args(
         "--tls12",
         help="scan for protocol version TLS1.2",
         action=utils.BooleanOptionalAction,
     )
 
 
-class ArgTls13(Plugin):
+class ArgTls13(plg.Plugin):
     """Argument for TLS1.3.
     """
 
-    config = ConfigItem("tls13", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("tls13", type=bool, default=None)
+    cli_args = plg.Args(
         "--tls13",
         help="scan for protocol version TLS1.3",
         action=utils.BooleanOptionalAction,
     )
 
 
-class GroupTlsVersions(Plugin):
+class GroupTlsVersions(plg.Plugin):
     """Argument group for TLS versions.
     """
 
-    group = Args(
+    group = plg.Args(
         title="TLS protocol versions",
         description=(
             "The following options specify the TLS protocol versions to scan. "
@@ -454,12 +483,12 @@ class GroupTlsVersions(Plugin):
             parser.error("at least one TLS version must be given")
 
 
-class ArgFeatures(Plugin):
+class ArgFeatures(plg.Plugin):
     """Argument for Features.
     """
 
-    config = ConfigItem("features", type=bool, default=True)
-    cli_args = Args(
+    config = structs.ConfigItem("features", type=bool, default=True)
+    cli_args = plg.Args(
         "--features",
         help=(
             "specifies whether to include or exclude all features in the scan. "
@@ -472,156 +501,156 @@ class ArgFeatures(Plugin):
     )
 
 
-class ArgCompression(Plugin):
+class ArgCompression(plg.Plugin):
     """Argument for compression feature.
     """
 
-    config = ConfigItem("compression", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("compression", type=bool, default=None)
+    cli_args = plg.Args(
         "--compression",
         help="scan for compression support",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanCompression]
+    workers = [compression.ScanCompression]
 
 
-class ArgDhGroup(Plugin):
+class ArgDhGroup(plg.Plugin):
     """Argument for DH groups.
     """
 
-    config = ConfigItem("dh_groups", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("dh_groups", type=bool, default=None)
+    cli_args = plg.Args(
         "--dh-groups",
         help="scan for finite field DH groups (only TL1.0 - TLS1.2)",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanDhGroups]
+    workers = [dh_params.ScanDhGroups]
 
 
-class ArgEncThenMac(Plugin):
+class ArgEncThenMac(plg.Plugin):
     """Argument for encrypt-then-mac extension.
     """
 
-    config = ConfigItem("encrypt_then_mac", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("encrypt_then_mac", type=bool, default=None)
+    cli_args = plg.Args(
         "--encrypt-then-mac",
         help="scan for encrypt-then-mac support (only TL1.0 - TLS1.2)",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanEncryptThenMac]
+    workers = [encrypt_then_mac.ScanEncryptThenMac]
 
 
-class ArgEphemKeyReuse(Plugin):
+class ArgEphemKeyReuse(plg.Plugin):
     """Argument for ephemeral key reuse.
     """
 
-    config = ConfigItem("ephemeral_key_reuse", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("ephemeral_key_reuse", type=bool, default=None)
+    cli_args = plg.Args(
         "--ephemeral-key-reuse",
         help="scan for reuse of ephemeral keys",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanEphemeralKeyReuse]
+    workers = [ephemeral_key_reuse.ScanEphemeralKeyReuse]
 
 
-class ArgExtMasterSecret(Plugin):
+class ArgExtMasterSecret(plg.Plugin):
     """Argument for extended-master-secret extension.
     """
 
-    config = ConfigItem("ext_master_secret", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("ext_master_secret", type=bool, default=None)
+    cli_args = plg.Args(
         "--ext-master-secret",
         help="scan for extended master secret support (only TL1.0 - TLS1.2)",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanExtendedMasterSecret]
+    workers = [master_secret.ScanExtendedMasterSecret]
 
 
-class ArgFallback(Plugin):
+class ArgFallback(plg.Plugin):
     """Argument for fallback.
     """
 
-    config = ConfigItem("fallback", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("fallback", type=bool, default=None)
+    cli_args = plg.Args(
         "--fallback",
         help="scan for downgrade attack prevention (TLS_FALLBACK_SCSV)",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanDowngrade]
+    workers = [downgrade.ScanDowngrade]
 
 
-class ArgGrease(Plugin):
+class ArgGrease(plg.Plugin):
     """Argument for GREASE.
     """
 
-    config = ConfigItem("grease", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("grease", type=bool, default=None)
+    cli_args = plg.Args(
         "--grease",
         help="scan for unknown parameter tolerance",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanGrease]
+    workers = [grease.ScanGrease]
 
 
-class ArgHeartbeat(Plugin):
+class ArgHeartbeat(plg.Plugin):
     """Argument for heartbeat extension.
     """
 
-    config = ConfigItem("heartbeat", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("heartbeat", type=bool, default=None)
+    cli_args = plg.Args(
         "--heartbeat",
         help="scan for heartbeat support",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanHeartbeat]
+    workers = [heartbeat.ScanHeartbeat]
 
 
-class ArgOcspStapling(Plugin):
+class ArgOcspStapling(plg.Plugin):
     """Argument for OCSP stapling.
     """
 
-    config = ConfigItem("ocsp_stapling", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("ocsp_stapling", type=bool, default=None)
+    cli_args = plg.Args(
         "--ocsp-stapling",
         help="scan for OCSP stapling support",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanOcspStapling]
+    workers = [ocsp_stapling.ScanOcspStapling]
 
 
-class ArgRenegotiation(Plugin):
+class ArgRenegotiation(plg.Plugin):
     """Argument for renegotiation feature.
     """
 
-    config = ConfigItem("renegotiation", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("renegotiation", type=bool, default=None)
+    cli_args = plg.Args(
         "--renegotiation",
         help="scan for renegotiation support (SSL30 - TLS1.2)",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanRenegotiation]
+    workers = [renegotiation.ScanRenegotiation]
 
 
-class ArgResumption(Plugin):
+class ArgResumption(plg.Plugin):
     """Argument for resumption feature.
     """
 
-    config = ConfigItem("resumption", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("resumption", type=bool, default=None)
+    cli_args = plg.Args(
         "--resumption",
         help=(
             "scan for resumption support (SSL30 - TLS1.2) and for PSK support (TLS1.3)"
         ),
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanResumption]
+    workers = [resumption.ScanResumption]
 
 
-class GroupFeatures(Plugin):
+class GroupFeatures(plg.Plugin):
     """Argument group for features.
     """
 
-    group = Args(title="Feature to include into the scan")
+    group = plg.Args(title="Feature to include into the scan")
     plugins = [
         ArgFeatures,
         ArgCompression,
@@ -652,15 +681,15 @@ class GroupFeatures(Plugin):
                 config.set(feat.config.name, default)
                 if default and feat.workers:
                     for worker in feat.workers:
-                        WorkManager.register(worker)
+                        plg.WorkManager.register(worker)
 
 
-class ArgVulnerabilities(Plugin):
+class ArgVulnerabilities(plg.Plugin):
     """Argument for Vulnerabilities.
     """
 
-    config = ConfigItem("vulnerabilities", type=bool, default=True)
-    cli_args = Args(
+    config = structs.ConfigItem("vulnerabilities", type=bool, default=True)
+    cli_args = plg.Args(
         "--vulnerabilities",
         help=(
             "specifies whether to include or exclude all vulnerabilities in the "
@@ -672,51 +701,51 @@ class ArgVulnerabilities(Plugin):
     )
 
 
-class ArgCcsInjection(Plugin):
+class ArgCcsInjection(plg.Plugin):
     """Argument for CCS-injection vulnerability.
     """
 
-    config = ConfigItem("ccs_injection", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("ccs_injection", type=bool, default=None)
+    cli_args = plg.Args(
         "--ccs-injection",
         help="scan for vulnerability CCS-injection (only TL1.0 - TLS1.2)",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanCcsInjection]
+    workers = [ccs_injection.ScanCcsInjection]
 
 
-class ArgHeartbleed(Plugin):
+class ArgHeartbleed(plg.Plugin):
     """Argument for heartbleed vulnerability.
     """
 
-    config = ConfigItem("heartbleed", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("heartbleed", type=bool, default=None)
+    cli_args = plg.Args(
         "--heartbleed",
         help="scan for the Heartbleed vulnerability CVE-2014-0160",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanHeartbleed]
+    workers = [heartbleed.ScanHeartbleed]
 
 
-class ArgPaddingOracle(Plugin):
+class ArgPaddingOracle(plg.Plugin):
     """Argument for CBC padding oracles.
     """
 
-    config = ConfigItem("padding_oracle", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("padding_oracle", type=bool, default=None)
+    cli_args = plg.Args(
         "--padding-oracle",
         help="scan for CBC padding oracles",
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanPaddingOracle]
+    workers = [padding_oracle.ScanPaddingOracle]
 
 
-class ArgPaddingOracleAccuracy(Plugin):
+class ArgPaddingOracleAccuracy(plg.Plugin):
     """Argument for oracle accuracy.
     """
 
-    config = ConfigItem("oracle_accuracy", type=str, default="medium")
-    cli_args = Args(
+    config = structs.ConfigItem("oracle_accuracy", type=str, default="medium")
+    cli_args = plg.Args(
         "--oracle-accuracy",
         help=(
             "the accuracy of the scan for CBC padding oracles. "
@@ -733,26 +762,26 @@ class ArgPaddingOracleAccuracy(Plugin):
     )
 
 
-class ArgRobot(Plugin):
+class ArgRobot(plg.Plugin):
     """Argument for ROBOT vulnerability.
     """
 
-    config = ConfigItem("robot", type=bool, default=None)
-    cli_args = Args(
+    config = structs.ConfigItem("robot", type=bool, default=None)
+    cli_args = plg.Args(
         "--robot",
         help=(
             "scan for ROBOT vulnerability CVE-2017-13099, etc. (only TL1.0 - TLS1.2)"
         ),
         action=utils.BooleanOptionalAction,
     )
-    workers = [ScanRobot]
+    workers = [robot.ScanRobot]
 
 
-class GroupVulnerabilities(Plugin):
+class GroupVulnerabilities(plg.Plugin):
     """Argument group vulnerabilities.
     """
 
-    group = Args(title="Vulnerabilities to scan for")
+    group = plg.Args(title="Vulnerabilities to scan for")
     plugins = [
         ArgVulnerabilities,
         ArgCcsInjection,
@@ -781,19 +810,19 @@ class GroupVulnerabilities(Plugin):
                 config.set(vuln.config.name, default)
                 if default and vuln.workers:
                     for worker in vuln.workers:
-                        WorkManager.register(worker)
+                        plg.WorkManager.register(worker)
                         base_vuln = True
 
         if base_vuln:
-            WorkManager.register(ScanBaseVulnerabilities)
+            plg.WorkManager.register(base_vulnerabilities.ScanBaseVulnerabilities)
 
 
-@BaseCommand.extend
-class SubcommandScan(Plugin):
+@plg.BaseCommand.extend
+class SubcommandScan(plg.Plugin):
     """CLI plugin to perform a scan against a TLS server.
     """
 
-    subcommand = Args("scan", help="performs a TLS server scan")
+    subcommand = plg.Args("scan", help="performs a TLS server scan")
     plugins = [
         GroupBasicScan,
         GroupX509,
@@ -802,4 +831,10 @@ class SubcommandScan(Plugin):
         GroupVulnerabilities,
         GroupServerProfile,
     ]
-    workers = [ScanStart, ScanCipherSuites, ScanSupportedGroups, ScanSigAlgs, ScanEnd]
+    workers = [
+        scanner_info.ScanStart,
+        eval_cipher_suites.ScanCipherSuites,
+        supported_groups.ScanSupportedGroups,
+        sig_algo.ScanSigAlgs,
+        scanner_info.ScanEnd,
+    ]
