@@ -3,13 +3,16 @@
 """
 # import basic stuff
 import logging
+from typing import Any, Optional
 
 # import own stuff
-from tlsmate import tls
-from tlsmate import structs
-from tlsmate import pdu
-from tlsmate.record_layer_state import RecordLayerState
-from tlsmate.socket import Socket
+import tlsmate.config as conf
+import tlsmate.pdu as pdu
+import tlsmate.record_layer_state as record_layer_state
+import tlsmate.recorder as rec
+import tlsmate.socket as socket
+import tlsmate.structs as structs
+import tlsmate.tls as tls
 
 # import other stuff
 
@@ -18,25 +21,21 @@ class RecordLayer(object):
     """Class implementing the record layer.
     """
 
-    def __init__(self, tlsmate, l4_addr):
-        self._l4_addr = l4_addr
-        self._tlsmate = tlsmate
+    def __init__(self, config: conf.Configuration, recorder: rec.Recorder) -> None:
         self._send_buffer = bytearray()
         self._receive_buffer = bytearray()
         self._fragment_max_size = 4 * 4096
-        self._write_state = None
-        self._read_state = None
-        self._socket = Socket(tlsmate)
+        self._write_state: Optional[record_layer_state.RecordLayerState] = None
+        self._read_state: Optional[record_layer_state.RecordLayerState] = None
+        self._socket = socket.Socket(config=config, recorder=recorder)
         self._flush_each_fragment = False
-        self._recorder = tlsmate.recorder
         self._ssl2 = False
 
-    def _send_fragment(self, rl_msg, **kwargs):
+    def _send_fragment(self, rl_msg: structs.RecordLayerMsg, **kwargs: Any) -> None:
         """Protects a fragment and adds it to the send queue.
 
         Arguments:
-            rl_msg (:obj:`tlsmate.structs.RecordLayerMsg`): The record layer
-                message to be sent.
+            rl_msg: The record layer message to be sent.
         """
 
         if self._write_state is not None:
@@ -82,7 +81,7 @@ class RecordLayer(object):
                 )
             )
 
-    def send_message(self, message, **kwargs):
+    def send_message(self, message: structs.RecordLayerMsg, **kwargs: Any) -> None:
         """Does everything the record layer needs to do for sending a message.
 
         The message is fragmented and protected (i.e. encrypted and authenticated) if
@@ -95,7 +94,7 @@ class RecordLayer(object):
         flush method to do so.
 
         Arguments:
-            message (:obj:`tlsmate.structs.RecordLayerMsg`): The message to send.
+            message: The message to send.
         """
 
         if message and message.content_type is tls.ContentType.SSL2:
@@ -106,22 +105,22 @@ class RecordLayer(object):
         else:
             self._fragment(message, **kwargs)
 
-    def open_socket(self, l4_addr):
+    def open_socket(self, l4_addr: structs.TransportEndpoint) -> None:
         """Opens the socket
 
         Arguments:
-            l4_addr (:obj:`tlsmate.structs.TransportEndpoint`): the l4_addr
+            l4_addr : the l4_addr
         """
 
         self._socket.open_socket(l4_addr)
 
-    def close_socket(self):
+    def close_socket(self) -> None:
         """Closes the socket. Obviously.
         """
 
         self._socket.close_socket()
 
-    def flush(self):
+    def flush(self) -> None:
         """Send all fragments in the send queue.
 
         This function is useful if e.g. multiple handshake messages shall be sent
@@ -131,15 +130,16 @@ class RecordLayer(object):
         self._socket.sendall(self._send_buffer)
         self._send_buffer = bytearray()
 
-    def wait_rl_msg(self, timeout=5, **kwargs):
+    def wait_rl_msg(
+        self, timeout: int = 5, **kwargs: Any
+    ) -> Optional[structs.RecordLayerMsg]:
         """Wait for a record layer message to be received from the network.
 
         Arguments:
-            timeout (int): The timeout in seconds to wait for the message. This
+            timeout: The timeout in seconds to wait for the message. This
                 parameter is optional and defaults to 5 seconds.
 
         Returns:
-            :obj:`tlsmate.structs.RecordLayerMsg`:
             A complete record layer message. If a timeout occurs, None is returned.
 
         Raises:
@@ -170,10 +170,14 @@ class RecordLayer(object):
                 length &= 0x7FFF
 
         else:
-            content_type, offset = pdu.unpack_uint8(self._receive_buffer, 0)
-            content_type = tls.ContentType.val2enum(content_type, alert_on_failure=True)
-            version, offset = pdu.unpack_uint16(self._receive_buffer, offset)
-            version = tls.Version.val2enum(version, alert_on_failure=True)
+            ct_type, offset = pdu.unpack_uint8(  # type: ignore
+                self._receive_buffer, 0
+            )
+            content_type = tls.ContentType.val2enum(ct_type, alert_on_failure=True)
+            vers, offset = pdu.unpack_uint16(  # type: ignore
+                self._receive_buffer, offset
+            )
+            version = tls.Version.val2enum(vers, alert_on_failure=True)
             length, offset = pdu.unpack_uint16(self._receive_buffer, offset)
 
         while len(self._receive_buffer) < (length + rl_len):
@@ -198,7 +202,7 @@ class RecordLayer(object):
         else:
             return self._read_state.unprotect_msg(rl_msg, **kwargs)
 
-    def update_state(self, new_state):
+    def update_state(self, new_state: structs.StateUpdateParams) -> None:
         """Update the record layer state.
 
         I.e. sent or received fragments are encrypted and authenticated.
@@ -209,7 +213,7 @@ class RecordLayer(object):
                 keying material for the symmetric ciphers and other relevant elements.
         """
 
-        state = RecordLayerState(new_state)
+        state = record_layer_state.RecordLayerState(new_state)
         if new_state.is_write_state:
             self._write_state = state
             state_type = "WRITE"
