@@ -3,12 +3,12 @@
 """
 # import basic stuff
 import struct
+from typing import Any
 
 # import own stuff
-from tlsmate import pdu
-from tlsmate import tls
-from tlsmate import structs
-from tlsmate.exception import ServerMalfunction
+import tlsmate.pdu as pdu
+import tlsmate.structs as structs
+import tlsmate.tls as tls
 
 # import other stuff
 from cryptography.hazmat.primitives import hmac, hashes
@@ -23,7 +23,7 @@ class RecordLayerState(object):
             the record layer state.
     """
 
-    def __init__(self, param):
+    def __init__(self, param: structs.StateUpdateParams) -> None:
 
         self._keys = param.keys
         self._mac = param.mac
@@ -35,7 +35,9 @@ class RecordLayerState(object):
         self._cipher_object = None
         self._iv = param.keys.iv
         if self._cipher.c_type == tls.CipherType.STREAM:
-            cipher = Cipher(self._cipher.algo(self._keys.enc), mode=None)
+            cipher = Cipher(
+                self._cipher.algo(self._keys.enc), mode=None  # type: ignore
+            )
             if param.is_write_state:
                 self._cipher_object = cipher.encryptor()
 
@@ -318,13 +320,14 @@ class RecordLayerState(object):
             fragment=cipher.encrypt(nonce, fragment, aad),
         )
 
-    def protect_msg(self, rl_msg, **kwargs):
+    def protect_msg(
+        self, rl_msg: structs.RecordLayerMsg, **kwargs: Any
+    ) -> structs.RecordLayerMsg:
         """Protects a fragment.
 
         Arguments:
-            rl_msg (:obj:`tlsmate.structs.RecordLayerMsg`): The record layer
-                message to protect.
-            kwargs (dict): additional parameters which can be used to control CBC
+            rl_msg: The record layer message to protect.
+            kwargs: additional parameters which can be used to control CBC
                 padding oracle related behavior
 
         Returns:
@@ -352,11 +355,11 @@ class RecordLayerState(object):
             bytes: The fragment appended with the MAC.
 
         Raises:
-            :obj:`tlsmate.exception.ServerMalfunction`: If the MAC is incorrect.
+            :obj:`tlsmate.tls.ServerMalfunction`: If the MAC is incorrect.
         """
 
         if len(fragment) < self._mac.mac_len:
-            raise ServerMalfunction(tls.ServerIssue.RECORD_TOO_SHORT)
+            raise tls.ServerMalfunction(tls.ServerIssue.RECORD_TOO_SHORT)
 
         msg_len = len(fragment) - self._mac.mac_len
         mac_received = fragment[msg_len:]
@@ -393,7 +396,7 @@ class RecordLayerState(object):
             mac_calculated = mac.finalize()
 
         if mac_calculated != mac_received:
-            raise ServerMalfunction(tls.ServerIssue.RECORD_MAC_INVALID)
+            raise tls.ServerMalfunction(tls.ServerIssue.RECORD_MAC_INVALID)
 
         return msg
 
@@ -407,7 +410,7 @@ class RecordLayerState(object):
             bytes: the decoded fragment.
 
         Raises:
-            :obj:`tlsmate.exception.ServerMalfunction`: If padding errors are detected.
+            :obj:`tlsmate.tls.ServerMalfunction`: If padding errors are detected.
         """
 
         if self._version <= tls.Version.TLS10:
@@ -427,7 +430,7 @@ class RecordLayerState(object):
         pad = plain_text[-1]
         pad_start = len(plain_text) - pad - 1
         if pad_start < 0:
-            raise ServerMalfunction(tls.ServerIssue.RECORD_WRONG_PADDING_LENGTH)
+            raise tls.ServerMalfunction(tls.ServerIssue.RECORD_WRONG_PADDING_LENGTH)
 
         padding = plain_text[pad_start:]
         if padding_cb:
@@ -437,7 +440,7 @@ class RecordLayerState(object):
 
         if self._version is not tls.Version.SSL30:
             if (struct.pack("!B", pad) * (pad + 1)) != padding:
-                raise ServerMalfunction(tls.ServerIssue.RECORD_WRONG_PADDING_BYTES)
+                raise tls.ServerMalfunction(tls.ServerIssue.RECORD_WRONG_PADDING_BYTES)
 
         return plain_text
 
@@ -612,7 +615,7 @@ class RecordLayerState(object):
             The unprotected record layer message (plain text).
 
         Raises:
-            :obj:`tlsmate.exception.ServerMalfunction`: If padding errors are detected.
+            :obj:`tlsmate.tls.ServerMalfunction`: If padding errors are detected.
         """
 
         aad = (
@@ -637,7 +640,7 @@ class RecordLayerState(object):
             idx -= 1
 
         if idx < 0:
-            raise ServerMalfunction(tls.ServerIssue.RECORD_WRONG_PADDING_LENGTH)
+            raise tls.ServerMalfunction(tls.ServerIssue.RECORD_WRONG_PADDING_LENGTH)
 
         self._seq_nbr += 1
         return structs.RecordLayerMsg(
@@ -646,7 +649,9 @@ class RecordLayerState(object):
             fragment=decoded[:idx],
         )
 
-    def unprotect_msg(self, rl_msg, **kwargs):
+    def unprotect_msg(
+        self, rl_msg: structs.RecordLayerMsg, **kwargs
+    ) -> structs.RecordLayerMsg:
         """Unprotects a record layer message.
 
         The message is decrypted and the authentication is verified.
