@@ -12,15 +12,14 @@ were regarded as protected records.
 # import basic stuff
 
 # import own stuff
-from tlsmate import msg
-from tlsmate import tls
-from tlsmate.exception import TlsConnectionClosedError
-from tlsmate.plugin import Worker
+import tlsmate.msg as msg
+import tlsmate.plugin as plg
+import tlsmate.tls as tls
 
 # import other stuff
 
 
-class ScanCcsInjection(Worker):
+class ScanCcsInjection(plg.Worker):
     name = "ccsinjection"
     descr = "scan for CCS injection vulnerability (CVE-2014-0224)"
     prio = 40
@@ -48,10 +47,10 @@ class ScanCcsInjection(Worker):
                 conn.send(msg.ChangeCipherSpec)
 
                 try:
-                    alert = conn.wait(msg.Alert, timeout=2000)
-                    if isinstance(alert, msg.Alert) and (
-                        alert.description is tls.AlertDescription.BAD_RECORD_MAC
-                        or alert.description is tls.AlertDescription.DECRYPTION_FAILED
+                    rec_msg = conn.wait(msg.Alert, timeout=2000, fail_on_timeout=False)
+                    if isinstance(rec_msg, msg.Alert) and (
+                        rec_msg.description is tls.AlertDescription.BAD_RECORD_MAC
+                        or rec_msg.description is tls.AlertDescription.DECRYPTION_FAILED
                     ):
                         # vulnerable
                         status = tls.ScanState.TRUE
@@ -59,7 +58,7 @@ class ScanCcsInjection(Worker):
                     else:
                         status = tls.ScanState.FALSE
 
-                except TlsConnectionClosedError:
+                except tls.TlsConnectionClosedError:
                     status = tls.ScanState.FALSE
 
         self.server_profile.allocate_vulnerabilities()
