@@ -4,19 +4,19 @@
 # import basic stuff
 
 # import own stuff
-from tlsmate import msg
-from tlsmate import tls
-from tlsmate.plugin import Worker
+import tlsmate.msg as msg
+import tlsmate.plugin as plg
+import tlsmate.tls as tls
 
 # import other stuff
 
 
-class ScanCompression(Worker):
+class ScanCompression(plg.Worker):
     name = "compression"
     descr = "scan for compression support"
     prio = 30
 
-    def compression(self, version):
+    def _compression(self, version):
         self.server_profile.allocate_features()
         features = self.server_profile.features
         if not hasattr(features, "compression"):
@@ -48,4 +48,4 @@ class ScanCompression(Worker):
 
     def run(self):
         for version in self.server_profile.get_versions(exclude=[tls.Version.SSL20]):
-            self.compression(version)
+            self._compression(version)
