@@ -4,22 +4,25 @@
 # import basic stuff
 
 # import own stuff
-from tlsmate import tls
-from tlsmate import msg
-from tlsmate.plugin import Worker
+import tlsmate.msg as msg
+import tlsmate.plugin as plg
+import tlsmate.tls as tls
 
 # import other stuff
 
 
-class ScanDowngrade(Worker):
+class ScanDowngrade(plg.Worker):
     name = "downgrade"
     descr = "scan for downgrade attack prevention"
     prio = 30
 
     def run(self):
-
         versions = self.server_profile.get_versions()
-        if len(versions) < 2:
+        nbr_versions = len(versions)
+        if tls.Version.SSL20 in versions:
+            nbr_versions -= 1
+
+        if nbr_versions < 2:
             status = tls.ScanState.NA
 
         else:
