@@ -4,14 +4,14 @@
 # import basic stuff
 
 # import own stuff
-from tlsmate import tls
-from tlsmate import msg
-from tlsmate.plugin import Worker
+import tlsmate.msg as msg
+import tlsmate.plugin as plg
+import tlsmate.tls as tls
 
 # import other stuff
 
 
-class ScanHeartbeat(Worker):
+class ScanHeartbeat(plg.Worker):
     name = "heartbeat"
     descr = "scan for heartbeat support"
     prio = 30
@@ -42,20 +42,25 @@ class ScanHeartbeat(Worker):
                             req.payload_length = len(req.payload)
                             req.padding = b"\xff" * 16
                             conn.send(req)
-                            res = conn.wait(msg.HeartbeatResponse, timeout=2000)
-                            if res is None:
+                            rec_msg = conn.wait(
+                                msg.Any, timeout=2000, fail_on_timeout=False,
+                            )
+                            if rec_msg is None:
                                 state = tls.HeartbeatState.NOT_REPONDING
 
-                            else:
+                            elif isinstance(rec_msg, msg.HeartbeatResponse):
                                 if (
-                                    req.payload_length == res.payload_length
-                                    and req.payload == res.payload
+                                    req.payload_length == rec_msg.payload_length
+                                    and req.payload == rec_msg.payload
                                 ):
                                     state = tls.HeartbeatState.TRUE
 
                                 else:
                                     state = tls.HeartbeatState.WRONG_RESPONSE
 
+                            else:
+                                state = tls.HeartbeatState.UNEXPECTED_MESSAGE
+
                         else:
                             state = tls.HeartbeatState.FALSE
 
