@@ -4,13 +4,13 @@
 # import basic stuff
 
 # import own stuff
-from tlsmate import tls
-from tlsmate.plugin import Worker
+import tlsmate.plugin as plg
+import tlsmate.tls as tls
 
 # import other stuff
 
 
-class ScanRenegotiation(Worker):
+class ScanRenegotiation(plg.Worker):
     name = "renegotiation"
     descr = "scan for renegotiation support"
     prio = 30
