@@ -5,17 +5,18 @@
 import sys
 import enum
 from dataclasses import dataclass
+from typing import List, Callable, Dict, Optional, Any
 
 # import own stuff
-
-from tlsmate.plugin import Worker
-from tlsmate import tls
-from tlsmate import utils
-from tlsmate import pdu
-from tlsmate.version import __version__
+import tlsmate.pdu as pdu
+import tlsmate.plugin as plg
+import tlsmate.server_profile as server_profile
+import tlsmate.tls as tls
+import tlsmate.utils as utils
+import tlsmate.version as vers
 
 # import other stuff
-import colorama
+import colorama  # type: ignore
 
 
 class Color(tls.ExtendedEnum):
@@ -73,11 +74,11 @@ class Style:
     ERROR: FontStyle = FontStyle(color=Color.RED, bold=True)
 
 
-def merge_styles(styles):
+def merge_styles(styles: List[Style]) -> FontStyle:
     """Returns the "worst" style from a given list
 
     Arguments:
-        styles (list of styles): the list of style strings
+        styles: the list of style strings
     """
     for style in [Style.ERROR, Style.BAD, Style.SOSO, Style.NEUTRAL, Style.GOOD]:
         if style in styles:
@@ -86,16 +87,18 @@ def merge_styles(styles):
     raise ValueError("cannot merge styles")
 
 
-def get_dict_value(profile, *keys, default=None):
+def get_dict_value(
+    profile: Dict[str, Any], *keys: str, default: Optional[Any] = None
+) -> Any:
     """Save function to get an item from a nested dict
 
     Arguments:
-        profile (dict): the dict to start with
-        *keys (str): a list the keys to descent to the wanted item of the profile
-        default(any): the value to return if a key does not exist. Defaults to None.
+        profile: the dict to start with
+        *keys: a list the keys to descent to the wanted item of the profile
+        default: the value to return if a key does not exist. Defaults to None.
 
     Returns:
-        object: the item
+        the item
     """
 
     if not profile:
@@ -111,7 +114,7 @@ def get_dict_value(profile, *keys, default=None):
     return profile
 
 
-def get_style(profile, *keys):
+def get_style(profile: Dict[str, Any], *keys: str) -> FontStyle:
     """Get the style string from a nested dict, descending according to the keys
 
     The leaf must be in ["good", "neutral", "soso", "bad", "headline", "bold", "reset",
@@ -119,18 +122,20 @@ def get_style(profile, *keys):
     ANSI escape code.
 
     Arguments:
-        profile (dict): the nested dict
-        *keys (str): the keys applied in sequence to descent to the style
+        profile: the nested dict
+        *keys: the keys applied in sequence to descent to the style
 
     Returns:
-        str: ANSI escape code string. If anything fails, Style.ERROR is returned.
+        ANSI escape code string. If anything fails, Style.ERROR is returned.
     """
     return getattr(
         Style, get_dict_value(profile, *keys, default="error").upper(), Style.ERROR
     )
 
 
-def get_style_applied(txt, profile, *keys, **kwargs):
+def get_style_applied(
+    txt: str, profile: Dict[str, Any], *keys: str, **kwargs: Any
+) -> str:
     """Descent into a nested dict and apply the found style to the given text.
 
     Arguments:
@@ -145,7 +150,7 @@ def get_style_applied(txt, profile, *keys, **kwargs):
     return get_style(profile, *keys).decorate(txt, **kwargs)
 
 
-def get_styled_text(data, *path, **kwargs):
+def get_styled_text(data: Dict[str, Any], *path: str, **kwargs: Any) -> str:
     """Comfortable way to use common structure to apply a style to a text.
 
     The item determined by data and path must be a dict with the following structure:
@@ -157,11 +162,11 @@ def get_styled_text(data, *path, **kwargs):
     which related to the "style" item.
 
     Arguments:
-        data (dict): the nested dict to use
-        *keys (str): the keys to descent into the nested dict
+        data: the nested dict to use
+        *path: the keys to descent into the nested dict
 
     Returns:
-        str: the "txt" string decorated with the "style" style.
+        the "txt" string decorated with the "style" style.
     """
     prof = get_dict_value(data, *path)
     if not prof:
@@ -170,16 +175,17 @@ def get_styled_text(data, *path, **kwargs):
     return get_style_applied(get_dict_value(prof, "txt"), prof, "style", **kwargs)
 
 
-def get_cert_ext(cert, name):
+def get_cert_ext(
+    cert: server_profile.SPCertificate, name: str
+) -> Optional[server_profile.SPCertExtension]:
     """Extract the given extension from a certificate.
 
     Arguments:
-        cert (:obj:`tlsmate.server_profile.SPCertificate`): the certificate object
-        name (str): the name of the extension
+        cert: the certificate object
+        name: the name of the extension
 
     Returns:
-        :obj:`tlsmate.server_profile.SPCertExtension`: the extension or None if not
-        found
+        the extension or None if not found
     """
     if not hasattr(cert, "extensions"):
         return None
@@ -191,7 +197,7 @@ def get_cert_ext(cert, name):
     return None
 
 
-class TextProfileWorker(Worker):
+class TextProfileWorker(plg.Worker):
     """Worker class which serializes a server profile.
     """
 
@@ -199,10 +205,10 @@ class TextProfileWorker(Worker):
     descr = "dump the scan results"
     prio = 1002
 
-    _callbacks = []
+    _callbacks: List[Callable[["TextProfileWorker"], None]] = []
 
     @classmethod
-    def augment_output(cls, callback):
+    def augment_output(cls, callback: Callable) -> Callable:
         """Decorator which can be used to register additional callbacks.
 
         Arguments:
@@ -270,7 +276,7 @@ class TextProfileWorker(Worker):
         print(Style.HEADLINE.decorate("A TLS configuration scanner (and more)"))
         print()
         table = utils.Table(indent=2, sep="  ")
-        table.row("tlsmate version", __version__)
+        table.row("tlsmate version", vers.__version__)
         table.row("repository", "https://gitlab.com/guballa/tlsmate")
         table.dump()
         print(
@@ -326,6 +332,10 @@ class TextProfileWorker(Worker):
                 addresses = ", ".join(host_info.name_resolution.ipv6_addresses)
                 table.row("IPv6 addresses", addresses)
 
+        proxy = getattr(host_info, "proxy", None)
+        if proxy:
+            table.row("HTTP-proxy", proxy)
+
         table.dump()
         print()
 
